@@ -11,8 +11,8 @@
 { REG_BADBR,	"REG_BADBR",	"invalid repetition count(s)" },
 { REG_ERANGE,	"REG_ERANGE",	"invalid character range" },
 { REG_ESPACE,	"REG_ESPACE",	"out of memory" },
-{ REG_BADRPT,	"REG_BADRPT",	"quantifier operand invalid" },
-{ REG_ASSERT,	"REG_ASSERT",	"\"can't happen\" -- you found a bug" },
+{ REG_BADRPT,	"REG_BADRPT",	"invalid quantifier operand" },
+{ REG_ASSERT,	"REG_ASSERT",	"\"cannot happen\" -- you found a bug" },
 { REG_INVARG,	"REG_INVARG",	"invalid argument to regex function" },
 { REG_MIXED,	"REG_MIXED",	"character widths of regex and string differ" },
 { REG_BADOPT,	"REG_BADOPT",	"invalid embedded option" },
