@@ -1,7 +1,7 @@
 /*
  * re_*exec and friends - match REs
  *
- * Copyright (c) 1998, 1999 Henry Spencer.  All rights reserved.
+ * Copyright © 1998, 1999 Henry Spencer.  All rights reserved.
  *
  * Development of this software was funded, in part, by Cray Research Inc.,
  * UUNET Communications Services Inc., Sun Microsystems Inc., and Scriptics
@@ -44,7 +44,7 @@ struct sset {			/* state set */
     unsigned hash;		/* hash of bitvector */
 #define	HASH(bv, nw)	(((nw) == 1) ? *(bv) : hash(bv, nw))
 #define	HIT(h,bv,ss,nw)	((ss)->hash == (h) && ((nw) == 1 || \
-	memcmp(VS(bv), VS((ss)->states), (nw)*sizeof(unsigned)) == 0))
+	memcmp((void*)(bv), (void*)((ss)->states), (nw)*sizeof(unsigned)) == 0))
     int flags;
 #define	STARTER		01	/* the initial state set */
 #define	POSTSTATE	02	/* includes the goal state */
@@ -57,11 +57,12 @@ struct sset {			/* state set */
 };
 
 struct dfa {
-    int nssets;			/* size of cache */
-    int nssused;		/* how many entries occupied yet */
-    int nstates;		/* number of states */
+    size_t nssets;			/* size of cache */
+    size_t nssused;		/* how many entries occupied yet */
+    size_t nstates;		/* number of states */
+    size_t wordsper;		/* length of state-set bitvectors */
     int ncolors;		/* length of outarc and inchain vectors */
-    int wordsper;		/* length of state-set bitvectors */
+    int cptsmalloced;		/* were the areas individually malloced? */
     struct sset *ssets;		/* state-set cache */
     unsigned *statesarea;	/* bitvector storage */
     unsigned *work;		/* pointer to work area within statesarea */
@@ -72,7 +73,6 @@ struct dfa {
     chr *lastpost;		/* location of last cache-flushed success */
     chr *lastnopr;		/* location of last cache-flushed NOPROGRESS */
     struct sset *search;	/* replacement-search-pointer memory */
-    int cptsmalloced;		/* were the areas individually malloced? */
     char *mallocarea;		/* self, or malloced area, or NULL */
 };
 
@@ -91,7 +91,6 @@ struct smalldfa {
     struct sset *outsarea[FEWSTATES*2 * FEWCOLORS];
     struct arcp incarea[FEWSTATES*2 * FEWCOLORS];
 };
-#define	DOMALLOC	((struct smalldfa *)NULL)	/* force malloc */
 
 /*
  * Internal variables, bundled for easy passing around.
@@ -117,7 +116,7 @@ struct vars {
 #define	ERR(e)	VERR(v, e)	/* record an error */
 #define	NOERR()	{if (ISERR()) return v->err;}	/* if error seen, return it */
 #define	OFF(p)	((p) - v->start)
-#define	LOFF(p)	((long)OFF(p))
+#define	LOFF(p)	((size_t)OFF(p))
 
 /*
  * forward declarations
@@ -146,7 +145,7 @@ static chr *shortest(struct vars *const, struct dfa *const, chr *const, chr *con
 static chr *lastCold(struct vars *const, struct dfa *const);
 static struct dfa *newDFA(struct vars *const, struct cnfa *const, struct colormap *const, struct smalldfa *);
 static void freeDFA(struct dfa *const);
-static unsigned hash(unsigned *const, const int);
+static unsigned hash(unsigned *const, int);
 static struct sset *initialize(struct vars *const, struct dfa *const, chr *const);
 static struct sset *miss(struct vars *const, struct dfa *const, struct sset *const, const pcolor, chr *const, chr *const);
 static int checkLAConstraint(struct vars *const, struct cnfa *const, chr *const, const pcolor);
@@ -172,8 +171,8 @@ exec(
 {
     AllocVars(v);
     int st, backref;
-    size_t n;
-    size_t i;
+    int n;
+    int i;
 #define	LOCALMAT	20
     regmatch_t mat[LOCALMAT];
 #define LOCALDFAS	40
@@ -187,10 +186,6 @@ exec(
 	FreeVars(v);
 	return REG_INVARG;
     }
-    if (re->re_csize != sizeof(chr)) {
-	FreeVars(v);
-	return REG_MIXED;
-    }
 
     /*
      * Setup.
@@ -236,15 +231,16 @@ exec(
     v->stop = (chr *)string + len;
     v->err = 0;
     assert(v->g->ntree >= 0);
-    n = (size_t) v->g->ntree;
+    n = v->g->ntree;
     if (n <= LOCALDFAS) {
 	v->subdfas = subdfas;
     } else {
 	v->subdfas = (struct dfa **) MALLOC(n * sizeof(struct dfa *));
     }
     if (v->subdfas == NULL) {
-	if (v->pmatch != pmatch && v->pmatch != mat)
+	if (v->pmatch != pmatch && v->pmatch != mat) {
 	    FREE(v->pmatch);
+	}
 	FreeVars(v);
 	return REG_ESPACE;
     }
@@ -269,7 +265,7 @@ exec(
     if (st == REG_OKAY && v->pmatch != pmatch && nmatch > 0) {
 	zapallsubs(pmatch, nmatch);
 	n = (nmatch < v->nmatch) ? nmatch : v->nmatch;
-	memcpy(VS(pmatch), VS(v->pmatch), n*sizeof(regmatch_t));
+	memcpy((void*)(pmatch), (void*)(v->pmatch), n*sizeof(regmatch_t));
     }
 
     /*
@@ -279,13 +275,15 @@ exec(
     if (v->pmatch != pmatch && v->pmatch != mat) {
 	FREE(v->pmatch);
     }
-    n = (size_t) v->g->ntree;
+    n = v->g->ntree;
     for (i = 0; i < n; i++) {
-	if (v->subdfas[i] != NULL)
+	if (v->subdfas[i] != NULL) {
 	    freeDFA(v->subdfas[i]);
+	}
     }
-    if (v->subdfas != subdfas)
+    if (v->subdfas != subdfas) {
 	FREE(v->subdfas);
+    }
     FreeVars(v);
     return st;
 }
@@ -300,9 +298,10 @@ getsubdfa(struct vars * v,
 	  struct subre * t)
 {
     if (v->subdfas[t->id] == NULL) {
-	v->subdfas[t->id] = newDFA(v, &t->cnfa, &v->g->cmap, DOMALLOC);
-	if (ISERR())
+	v->subdfas[t->id] = newDFA(v, &t->cnfa, &v->g->cmap, NULL);
+	if (ISERR()) {
 	    return NULL;
+	}
     }
     return v->subdfas[t->id];
 }
@@ -332,7 +331,7 @@ simpleFind(
     s = newDFA(v, &v->g->search, cm, &v->dfa1);
     assert(!(ISERR() && s != NULL));
     NOERR();
-    MDEBUG(("\nsearch at %ld\n", LOFF(v->start)));
+    MDEBUG(("\nsearch at %" TCL_Z_MODIFIER "u\n", LOFF(v->start)));
     cold = NULL;
     close = shortest(v, s, v->start, v->start, v->stop, &cold, NULL);
     freeDFA(s);
@@ -360,12 +359,12 @@ simpleFind(
     assert(cold != NULL);
     open = cold;
     cold = NULL;
-    MDEBUG(("between %ld and %ld\n", LOFF(open), LOFF(close)));
+    MDEBUG(("between %" TCL_Z_MODIFIER "u and %" TCL_Z_MODIFIER "u\n", LOFF(open), LOFF(close)));
     d = newDFA(v, cnfa, cm, &v->dfa1);
     assert(!(ISERR() && d != NULL));
     NOERR();
     for (begin = open; begin <= close; begin++) {
-	MDEBUG(("\nfind trying at %ld\n", LOFF(begin)));
+	MDEBUG(("\nfind trying at %" TCL_Z_MODIFIER "u\n", LOFF(begin)));
 	if (shorter) {
 	    end = shortest(v, d, begin, begin, v->stop, NULL, &hitend);
 	} else {
@@ -476,7 +475,7 @@ complicatedFindLoop(
     cold = NULL;
     close = v->start;
     do {
-	MDEBUG(("\ncsearch at %ld\n", LOFF(close)));
+	MDEBUG(("\ncsearch at %" TCL_Z_MODIFIER "u\n", LOFF(close)));
 	close = shortest(v, s, close, close, v->stop, &cold, NULL);
 	if (close == NULL) {
 	    break;		/* NOTE BREAK */
@@ -484,9 +483,9 @@ complicatedFindLoop(
 	assert(cold != NULL);
 	open = cold;
 	cold = NULL;
-	MDEBUG(("cbetween %ld and %ld\n", LOFF(open), LOFF(close)));
+	MDEBUG(("cbetween %" TCL_Z_MODIFIER "u and %" TCL_Z_MODIFIER "u\n", LOFF(open), LOFF(close)));
 	for (begin = open; begin <= close; begin++) {
-	    MDEBUG(("\ncomplicatedFind trying at %ld\n", LOFF(begin)));
+	    MDEBUG(("\ncomplicatedFind trying at %" TCL_Z_MODIFIER "u\n", LOFF(begin)));
 	    estart = begin;
 	    estop = v->stop;
 	    for (;;) {
@@ -502,7 +501,7 @@ complicatedFindLoop(
 		    break;	/* NOTE BREAK OUT */
 		}
 
-		MDEBUG(("tentative end %ld\n", LOFF(end)));
+		MDEBUG(("tentative end %" TCL_Z_MODIFIER "u\n", LOFF(end)));
 		zapallsubs(v->pmatch, v->nmatch);
 		er = cdissect(v, v->g->tree, begin, end);
 		if (er == REG_OKAY) {
@@ -551,8 +550,8 @@ zapallsubs(
     size_t i;
 
     for (i = n-1; i > 0; i--) {
-	p[i].rm_so = -1;
-	p[i].rm_eo = -1;
+	p[i].rm_so = FREESTATE;
+	p[i].rm_eo = FREESTATE;
     }
 }
 
@@ -566,11 +565,11 @@ zaptreesubs(
     struct subre *const t)
 {
     if (t->op == '(') {
-	int n = t->subno;
+	size_t n = t->subno;
 	assert(n > 0);
-	if ((size_t) n < v->nmatch) {
-	    v->pmatch[n].rm_so = -1;
-	    v->pmatch[n].rm_eo = -1;
+	if (n < v->nmatch) {
+	    v->pmatch[n].rm_so = FREESTATE;
+	    v->pmatch[n].rm_eo = FREESTATE;
 	}
     }
 
@@ -629,7 +628,7 @@ cdissect(
     int er;
 
     assert(t != NULL);
-    MDEBUG(("cdissect %ld-%ld %c\n", LOFF(begin), LOFF(end), t->op));
+    MDEBUG(("cdissect %" TCL_Z_MODIFIER "u-%" TCL_Z_MODIFIER "u %c\n", LOFF(begin), LOFF(end), t->op));
 
     switch (t->op) {
     case '=':			/* terminal node */
@@ -716,7 +715,7 @@ ccondissect(
     if (mid == NULL) {
 	return REG_NOMATCH;
     }
-    MDEBUG(("tentative midpoint %ld\n", LOFF(mid)));
+    MDEBUG(("tentative midpoint %" TCL_Z_MODIFIER "u\n", LOFF(mid)));
 
     /*
      * Iterate until satisfaction or failure.
@@ -767,7 +766,7 @@ ccondissect(
 	    MDEBUG(("%d failed midpoint\n", t->id));
 	    return REG_NOMATCH;
 	}
-	MDEBUG(("%d: new midpoint %ld\n", t->id, LOFF(mid)));
+	MDEBUG(("%d: new midpoint %" TCL_Z_MODIFIER "u\n", t->id, LOFF(mid)));
 	zaptreesubs(v, t->left);
 	zaptreesubs(v, t->right);
     }
@@ -807,7 +806,7 @@ crevcondissect(
     if (mid == NULL) {
 	return REG_NOMATCH;
     }
-    MDEBUG(("tentative midpoint %ld\n", LOFF(mid)));
+    MDEBUG(("tentative midpoint %" TCL_Z_MODIFIER "u\n", LOFF(mid)));
 
     /*
      * Iterate until satisfaction or failure.
@@ -858,7 +857,7 @@ crevcondissect(
 	    MDEBUG(("%d failed midpoint\n", t->id));
 	    return REG_NOMATCH;
 	}
-	MDEBUG(("%d: new midpoint %ld\n", t->id, LOFF(mid)));
+	MDEBUG(("%d: new midpoint %" TCL_Z_MODIFIER "u\n", t->id, LOFF(mid)));
 	zaptreesubs(v, t->left);
 	zaptreesubs(v, t->right);
     }
@@ -890,7 +889,7 @@ cbrdissect(
     MDEBUG(("cbackref n%d %d{%d-%d}\n", t->id, n, min, max));
 
     /* get the backreferenced string */
-    if (v->pmatch[n].rm_so == -1) {
+    if (v->pmatch[n].rm_so == FREESTATE) {
 	return REG_NOMATCH;
     }
     brstring = v->start + v->pmatch[n].rm_so;
@@ -924,17 +923,20 @@ cbrdissect(
 
     assert(end > begin);
     tlen = end - begin;
-    if (tlen % brlen != 0)
+    if (tlen % brlen != 0) {
 	return REG_NOMATCH;
+    }
     numreps = tlen / brlen;
-    if (numreps < (size_t)min || (numreps > (size_t)max && max != DUPINF))
+    if (numreps < (size_t)min || (numreps > (size_t)max && max != DUPINF)) {
 	return REG_NOMATCH;
+    }
 
     /* okay, compare the actual string contents */
     p = begin;
     while (numreps-- > 0) {
-	if ((*v->g->compare) (brstring, p, brlen) != 0)
+	if ((*v->g->compare) (brstring, p, brlen) != 0) {
 	    return REG_NOMATCH;
+	}
 	p += brlen;
     }
 
@@ -1011,8 +1013,9 @@ citerdissect(struct vars * v,
      */
     min_matches = t->min;
     if (min_matches <= 0) {
-	if (begin == end)
+	if (begin == end) {
 	    return REG_OKAY;
+	}
 	min_matches = 1;
     }
 
@@ -1026,8 +1029,9 @@ citerdissect(struct vars * v,
      * sub-match endpoints in endpts[1..max_matches].
      */
     max_matches = end - begin;
-    if (max_matches > (size_t)t->max && t->max != DUPINF)
+    if (max_matches > (size_t)t->max && t->max != DUPINF) {
 	max_matches = t->max;
+    }
     if (max_matches < (size_t)min_matches)
 	max_matches = min_matches;
     endpts = (chr **) MALLOC((max_matches + 1) * sizeof(chr *));
@@ -1066,12 +1070,13 @@ citerdissect(struct vars * v,
 	    k--;
 	    goto backtrack;
 	}
-	MDEBUG(("%d: working endpoint %d: %ld\n",
+	MDEBUG(("%d: working endpoint %d: %" TCL_Z_MODIFIER "u\n",
 		t->id, k, LOFF(endpts[k])));
 
 	/* k'th sub-match can no longer be considered verified */
-	if (nverified >= k)
+	if (nverified >= k) {
 	    nverified = k - 1;
+	}
 
 	if (endpts[k] != end) {
 	    /* haven't reached end yet, try another iteration if allowed */
@@ -1097,8 +1102,9 @@ citerdissect(struct vars * v,
 	 * number of matches, start the slow part: recurse to verify each
 	 * sub-match.  We always have k <= max_matches, needn't check that.
 	 */
-	if (k < min_matches)
+	if (k < min_matches) {
 	    goto backtrack;
+	}
 
 	MDEBUG(("%d: verifying %d..%d\n", t->id, nverified + 1, k));
 
@@ -1109,8 +1115,9 @@ citerdissect(struct vars * v,
 		nverified = i;
 		continue;
 	    }
-	    if (er == REG_NOMATCH)
+	    if (er == REG_NOMATCH) {
 		break;
+	    }
 	    /* oops, something failed */
 	    FREE(endpts);
 	    return er;
@@ -1184,8 +1191,9 @@ creviterdissect(struct vars * v,
      */
     min_matches = t->min;
     if (min_matches <= 0) {
-	if (begin == end)
+	if (begin == end) {
 	    return REG_OKAY;
+	}
 	min_matches = 1;
     }
 
@@ -1239,8 +1247,9 @@ creviterdissect(struct vars * v,
 	    limit++;
 
 	/* if this is the last allowed sub-match, it must reach to the end */
-	if ((size_t)k >= max_matches)
+	if ((size_t)k >= max_matches) {
 	    limit = end;
+	}
 
 	/* try to find an endpoint for the k'th sub-match */
 	endpts[k] = shortest(v, d, endpts[k - 1], limit, end,
@@ -1250,12 +1259,13 @@ creviterdissect(struct vars * v,
 	    k--;
 	    goto backtrack;
 	}
-	MDEBUG(("%d: working endpoint %d: %ld\n",
+	MDEBUG(("%d: working endpoint %d: %" TCL_Z_MODIFIER "u\n",
 		t->id, k, LOFF(endpts[k])));
 
 	/* k'th sub-match can no longer be considered verified */
-	if (nverified >= k)
+	if (nverified >= k) {
 	    nverified = k - 1;
+	}
 
 	if (endpts[k] != end) {
 	    /* haven't reached end yet, try another iteration if allowed */
@@ -1276,8 +1286,9 @@ creviterdissect(struct vars * v,
 	 * number of matches, start the slow part: recurse to verify each
 	 * sub-match.  We always have k <= max_matches, needn't check that.
 	 */
-	if (k < min_matches)
+	if (k < min_matches) {
 	    goto backtrack;
+	}
 
 	MDEBUG(("%d: verifying %d..%d\n", t->id, nverified + 1, k));
 
@@ -1288,8 +1299,9 @@ creviterdissect(struct vars * v,
 		nverified = i;
 		continue;
 	    }
-	    if (er == REG_NOMATCH)
+	    if (er == REG_NOMATCH) {
 		break;
+	    }
 	    /* oops, something failed */
 	    FREE(endpts);
 	    return er;
