@@ -354,7 +354,7 @@ FreeElements(
 	for (i=0; i<len; i++) {
 	    Tcl_DecrRefCount(arithSeriesRepPtr->elements[i]);
 	}
-	Tcl_Free((char *) arithSeriesRepPtr->elements);
+	Tcl_Free((void *)arithSeriesRepPtr->elements);
 	arithSeriesRepPtr->elements = NULL;
     }
 }
@@ -368,7 +368,7 @@ FreeArithSeriesInternalRep(
 
     if (arithSeriesRepPtr) {
 	FreeElements(arithSeriesRepPtr);
-	Tcl_Free((char *) arithSeriesRepPtr);
+	Tcl_Free((void *)arithSeriesRepPtr);
     }
 }
 
@@ -514,11 +514,11 @@ assignNumber(
 
     if (Tcl_GetNumberFromObj(interp, numberObj, &clientData,
 		&tcl_number_type) != TCL_OK) {
-    	return TCL_ERROR;
+	return TCL_ERROR;
     }
     if (tcl_number_type == TCL_NUMBER_BIG) {
-    	/* bignum is not supported yet. */
-    	Tcl_WideInt w;
+	/* bignum is not supported yet. */
+	Tcl_WideInt w;
 	(void)Tcl_GetWideIntFromObj(interp, numberObj, &w);
 	return TCL_ERROR;
     }
@@ -642,7 +642,7 @@ TclNewArithSeriesObj(
     if (len > TCL_SIZE_MAX) {
 	Tcl_SetObjResult(interp, Tcl_NewStringObj(
 		"max length of a Tcl list exceeded", TCL_AUTO_LENGTH));
-	Tcl_SetErrorCode(interp, "TCL", "MEMORY", (void *)NULL);
+	Tcl_SetErrorCode(interp, "TCL", "MEMORY", (char *)NULL);
 	return NULL;
     }
 
@@ -957,7 +957,7 @@ TclArithSeriesGetElements(
 			Tcl_SetObjResult(interp, Tcl_NewStringObj(
 				"max length of a Tcl list exceeded",
 				TCL_AUTO_LENGTH));
-			Tcl_SetErrorCode(interp, "TCL", "MEMORY", (void *)NULL);
+			Tcl_SetErrorCode(interp, "TCL", "MEMORY", (char *)NULL);
 		    }
 		    return TCL_ERROR;
 		}
@@ -982,7 +982,7 @@ TclArithSeriesGetElements(
 	if (interp != NULL) {
 	    Tcl_SetObjResult(interp, Tcl_NewStringObj(
 		    "value is not an arithseries", TCL_AUTO_LENGTH));
-	    Tcl_SetErrorCode(interp, "TCL", "VALUE", "UNKNOWN", (void *)NULL);
+	    Tcl_SetErrorCode(interp, "TCL", "VALUE", "UNKNOWN", (char *)NULL);
 	}
 	return TCL_ERROR;
     }
