@@ -4,7 +4,7 @@
  *	This file provides the facilities which allow Tcl and other packages
  *	to embed configuration information into their binary libraries.
  *
- * Copyright (c) 2002 Andreas Kupries <andreas_kupries@users.sourceforge.net>
+ * Copyright © 2002 Andreas Kupries <andreas_kupries@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -31,7 +31,7 @@
  * the (Tcl_Interp *) in which it is stored, and the encoding.
  */
 
-typedef struct QCCD {
+typedef struct {
     Tcl_Obj *pkg;
     Tcl_Interp *interp;
     char *encoding;
@@ -41,11 +41,10 @@ typedef struct QCCD {
  * Static functions in this file:
  */
 
-static Tcl_ObjCmdProc QueryConfigObjCmd;
-static void		QueryConfigDelete(ClientData clientData);
+static Tcl_ObjCmdProc		QueryConfigObjCmd;
+static Tcl_CmdDeleteProc	QueryConfigDelete;
+static Tcl_InterpDeleteProc	ConfigDictDeleteProc;
 static Tcl_Obj *	GetConfigDict(Tcl_Interp *interp);
-static void		ConfigDictDeleteProc(ClientData clientData,
-			    Tcl_Interp *interp);
 
 /*
  *----------------------------------------------------------------------
@@ -77,11 +76,11 @@ Tcl_RegisterConfig(
     Tcl_Obj *pDB, *pkgDict;
     Tcl_DString cmdName;
     const Tcl_Config *cfg;
-    QCCD *cdPtr = (QCCD *)ckalloc(sizeof(QCCD));
+    QCCD *cdPtr = (QCCD *)Tcl_Alloc(sizeof(QCCD));
 
     cdPtr->interp = interp;
     if (valEncoding) {
-	cdPtr->encoding = (char *)ckalloc(strlen(valEncoding)+1);
+	cdPtr->encoding = (char *)Tcl_Alloc(strlen(valEncoding)+1);
 	strcpy(cdPtr->encoding, valEncoding);
     } else {
 	cdPtr->encoding = NULL;
@@ -128,7 +127,7 @@ Tcl_RegisterConfig(
      */
 
     for (cfg=configuration ; cfg->key!=NULL && cfg->key[0]!='\0' ; cfg++) {
-	Tcl_DictObjPut(interp, pkgDict, Tcl_NewStringObj(cfg->key, -1),
+	TclDictPut(interp, pkgDict, cfg->key,
 		Tcl_NewByteArrayObj((unsigned char *)cfg->value, strlen(cfg->value)));
     }
 
@@ -179,7 +178,7 @@ Tcl_RegisterConfig(
  * QueryConfigObjCmd --
  *
  *	Implementation of "::<package>::pkgconfig", the command to query
- *	configuration information embedded into a binary library.
+ *	configuration information embedded into a library.
  *
  * Results:
  *	A standard Tcl result.
@@ -192,7 +191,7 @@ Tcl_RegisterConfig(
 
 static int
 QueryConfigObjCmd(
-    ClientData clientData,
+    void *clientData,
     Tcl_Interp *interp,
     int objc,
     Tcl_Obj *const *objv)
@@ -200,13 +199,13 @@ QueryConfigObjCmd(
     QCCD *cdPtr = (QCCD *)clientData;
     Tcl_Obj *pkgName = cdPtr->pkg;
     Tcl_Obj *pDB, *pkgDict, *val, *listPtr;
-    int n, index;
+    Tcl_Size m, n = 0;
     static const char *const subcmdStrings[] = {
 	"get", "list", NULL
     };
     enum subcmds {
 	CFG_GET, CFG_LIST
-    };
+    } index;
     Tcl_DString conv;
     Tcl_Encoding venc = NULL;
     const char *value;
@@ -230,11 +229,11 @@ QueryConfigObjCmd(
 
 	Tcl_SetObjResult(interp, Tcl_NewStringObj("package not known", -1));
 	Tcl_SetErrorCode(interp, "TCL", "FATAL", "PKGCFG_BASE",
-		Tcl_GetString(pkgName), NULL);
+		TclGetString(pkgName), (char *)NULL);
 	return TCL_ERROR;
     }
 
-    switch ((enum subcmds) index) {
+    switch (index) {
     case CFG_GET:
 	if (objc != 3) {
 	    Tcl_WrongNumArgs(interp, 2, objv, "key");
@@ -245,7 +244,7 @@ QueryConfigObjCmd(
 		|| val == NULL) {
 	    Tcl_SetObjResult(interp, Tcl_NewStringObj("key not known", -1));
 	    Tcl_SetErrorCode(interp, "TCL", "LOOKUP", "CONFIG",
-		    Tcl_GetString(objv[2]), NULL);
+		    TclGetString(objv[2]), (char *)NULL);
 	    return TCL_ERROR;
 	}
 
@@ -259,7 +258,10 @@ QueryConfigObjCmd(
 	 * Value is stored as-is in a byte array, see Bug [9b2e636361],
 	 * so we have to decode it first.
 	 */
-	value = (const char *) Tcl_GetByteArrayFromObj(val, &n);
+	value = (const char *) Tcl_GetBytesFromObj(interp, val, &n);
+	if (value == NULL) {
+	    return TCL_ERROR;
+	}
 	value = Tcl_ExternalToUtfDString(venc, value, n, &conv);
 	Tcl_SetObjResult(interp, Tcl_NewStringObj(value,
 		Tcl_DStringLength(&conv)));
@@ -272,17 +274,17 @@ QueryConfigObjCmd(
 	    return TCL_ERROR;
 	}
 
-	Tcl_DictObjSize(interp, pkgDict, &n);
-	listPtr = Tcl_NewListObj(n, NULL);
+	Tcl_DictObjSize(interp, pkgDict, &m);
+	listPtr = Tcl_NewListObj(m, NULL);
 
 	if (!listPtr) {
 	    Tcl_SetObjResult(interp, Tcl_NewStringObj(
 		    "insufficient memory to create list", -1));
-	    Tcl_SetErrorCode(interp, "TCL", "MEMORY", NULL);
+	    Tcl_SetErrorCode(interp, "TCL", "MEMORY", (char *)NULL);
 	    return TCL_ERROR;
 	}
 
-	if (n) {
+	if (m) {
 	    Tcl_DictSearch s;
 	    Tcl_Obj *key;
 	    int done;
@@ -322,7 +324,7 @@ QueryConfigObjCmd(
 
 static void
 QueryConfigDelete(
-    ClientData clientData)
+    void *clientData)
 {
     QCCD *cdPtr = (QCCD *)clientData;
     Tcl_Obj *pkgName = cdPtr->pkg;
@@ -331,9 +333,9 @@ QueryConfigDelete(
     Tcl_DictObjRemove(NULL, pDB, pkgName);
     Tcl_DecrRefCount(pkgName);
     if (cdPtr->encoding) {
-	ckfree((char *)cdPtr->encoding);
+	Tcl_Free(cdPtr->encoding);
     }
-    ckfree((char *)cdPtr);
+    Tcl_Free(cdPtr);
 }
 
 /*
@@ -389,12 +391,10 @@ GetConfigDict(
 
 static void
 ConfigDictDeleteProc(
-    ClientData clientData,	/* Pointer to Tcl_Obj. */
-    Tcl_Interp *interp)		/* Interpreter being deleted. */
+    void *clientData,	/* Pointer to Tcl_Obj. */
+    TCL_UNUSED(Tcl_Interp *))
 {
-    Tcl_Obj *pDB = (Tcl_Obj *)clientData;
-
-    Tcl_DecrRefCount(pDB);
+    Tcl_DecrRefCount((Tcl_Obj *)clientData);
 }
 
 /*
