@@ -6,8 +6,8 @@
  *	commands ("events") before they are executed. Commands defined in
  *	history.tcl may be used to perform history substitutions.
  *
- * Copyright (c) 1990-1993 The Regents of the University of California.
- * Copyright (c) 1994-1997 Sun Microsystems, Inc.
+ * Copyright © 1990-1993 The Regents of the University of California.
+ * Copyright © 1994-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -58,30 +58,21 @@ Tcl_RecordAndEval(
     const char *cmd,		/* Command to record. */
     int flags)			/* Additional flags. TCL_NO_EVAL means only
 				 * record: don't execute command.
-				 * TCL_EVAL_GLOBAL means evaluate the script
-				 * in global variable context instead of the
-				 * current procedure. */
+				 * TCL_EVAL_GLOBAL means use Tcl_GlobalEval
+				 * instead of Tcl_Eval. */
 {
     Tcl_Obj *cmdPtr;
-    int length = strlen(cmd);
     int result;
 
-    if (length > 0) {
+    if (cmd[0]) {
 	/*
 	 * Call Tcl_RecordAndEvalObj to do the actual work.
 	 */
 
-	cmdPtr = Tcl_NewStringObj(cmd, length);
+	cmdPtr = Tcl_NewStringObj(cmd, -1);
 	Tcl_IncrRefCount(cmdPtr);
 	result = Tcl_RecordAndEvalObj(interp, cmdPtr, flags);
 
-	/*
-	 * Move the interpreter's object result to the string result, then
-	 * reset the object result.
-	 */
-
-	(void) Tcl_GetStringResult(interp);
-
 	/*
 	 * Discard the Tcl object created to hold the command.
 	 */
@@ -131,15 +122,15 @@ Tcl_RecordAndEvalObj(
 {
     int result, call = 1;
     Tcl_CmdInfo info;
-    HistoryObjs *histObjsPtr = (HistoryObjs *)
-	    Tcl_GetAssocData(interp, HISTORY_OBJS_KEY, NULL);
+    HistoryObjs *histObjsPtr =
+	    (HistoryObjs *)Tcl_GetAssocData(interp, HISTORY_OBJS_KEY, NULL);
 
     /*
      * Create the references to the [::history add] command if necessary.
      */
 
     if (histObjsPtr == NULL) {
-	histObjsPtr = (HistoryObjs *)ckalloc(sizeof(HistoryObjs));
+	histObjsPtr = (HistoryObjs *)Tcl_Alloc(sizeof(HistoryObjs));
 	TclNewLiteralStringObj(histObjsPtr->historyObj, "::history");
 	TclNewLiteralStringObj(histObjsPtr->addObj, "add");
 	Tcl_IncrRefCount(histObjsPtr->historyObj);
@@ -212,14 +203,14 @@ Tcl_RecordAndEvalObj(
 
 static void
 DeleteHistoryObjs(
-    ClientData clientData,
-    Tcl_Interp *interp)
+    void *clientData,
+    TCL_UNUSED(Tcl_Interp *))
 {
     HistoryObjs *histObjsPtr = (HistoryObjs *)clientData;
 
     TclDecrRefCount(histObjsPtr->historyObj);
     TclDecrRefCount(histObjsPtr->addObj);
-    ckfree(histObjsPtr);
+    Tcl_Free(histObjsPtr);
 }
 
 /*
