@@ -4,7 +4,7 @@
  *	This procedure provides a version of the TclpDlopen for use in
  *	systems that don't support dynamic loading; it just returns an error.
  *
- * Copyright (c) 1995-1997 Sun Microsystems, Inc.
+ * Copyright © 1995-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -53,36 +53,6 @@ TclpDlopen(
     return TCL_ERROR;
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TclGuessPackageName --
- *
- *	If the "load" command is invoked without providing a package name,
- *	this procedure is invoked to try to figure it out.
- *
- * Results:
- *	Always returns 0 to indicate that we couldn't figure out a package
- *	name; generic code will then try to guess the package from the file
- *	name. A return value of 1 would have meant that we figured out the
- *	package name and put it in bufPtr.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-int
-TclGuessPackageName(
-    const char *fileName,	/* Name of file containing package (already
-				 * translated to local form if needed). */
-    Tcl_DString *bufPtr)	/* Initialized empty dstring. Append package
-				 * name to this if possible. */
-{
-    return 0;
-}
-
 /*
  * These functions are fallbacks if we somehow determine that the platform can
  * do loading from memory but the user wishes to disable it. They just report
@@ -93,22 +63,20 @@ TclGuessPackageName(
 
 MODULE_SCOPE void *
 TclpLoadMemoryGetBuffer(
-     size_t size)			/* Dummy: unused by this implementation */
+    TCL_UNUSED(size_t))
 {
     return NULL;
 }
 
 MODULE_SCOPE int
 TclpLoadMemory(
-    void *buffer,		/* Dummy: unused by this implementation */
-    size_t size,		/* Dummy: unused by this implementation */
-    int codeSize,		/* Dummy: unused by this implementation */
-    const char *path,	/* Dummy: unused by this implementation */
-    Tcl_LoadHandle *loadHandle,	/* Dummy: unused by this implementation */
-    Tcl_FSUnloadFileProc **unloadProcPtr,
-				/* Dummy: unused by this implementation */
-    int flags)
-				/* Dummy: unused by this implementation */
+    TCL_UNUSED(void *),
+    TCL_UNUSED(size_t),
+    TCL_UNUSED(Tcl_Size),
+    TCL_UNUSED(const char *),
+    TCL_UNUSED(Tcl_LoadHandle *),
+    TCL_UNUSED(Tcl_FSUnloadFileProc **),
+    TCL_UNUSED(int))
 {
     return TCL_ERROR;
 }
