@@ -405,6 +405,8 @@ struct Foundation {
 				 * "<cloned>" pseudo-constructor. */
     Tcl_Obj *defineName;	/* Fully qualified name of oo::define. */
     Tcl_Obj *myName;		/* The "my" shared object. */
+    Tcl_Obj *mcdName;		/* The shared object for calling the helper to
+				 * mix in class delegates. */
 };
 
 /*
@@ -603,7 +605,7 @@ MODULE_SCOPE Tcl_Var	TclOOLookupObjectVar(Tcl_Interp *interp,
 MODULE_SCOPE int	TclNRObjectContextInvokeNext(Tcl_Interp *interp,
 			    Tcl_ObjectContext context, Tcl_Size objc,
 			    Tcl_Obj *const *objv, Tcl_Size skip);
-MODULE_SCOPE void	TclOONewBasicMethod(Class *clsPtr,
+MODULE_SCOPE void	TclOODefineBasicMethods(Class *clsPtr,
 			    const DeclaredClassMethod *dcm);
 MODULE_SCOPE Tcl_Obj *	TclOOObjectName(Tcl_Interp *interp, Object *oPtr);
 MODULE_SCOPE void	TclOOReleaseClassContents(Tcl_Interp *interp,
@@ -671,20 +673,26 @@ MODULE_SCOPE void	TclOORegisterInstanceProperty(Object *oPtr,
 /*
  * Convenience macros for iterating through hash tables. FOREACH_HASH_DECLS
  * sets up the declarations needed for the main macro, FOREACH_HASH, which
- * does the actual iteration. FOREACH_HASH_VALUE is a restricted version that
- * only iterates over values.
+ * does the actual iteration. FOREACH_HASH_KEY and FOREACH_HASH_VALUE are
+ * restricted versions that only iterate over keys or values respectively.
  * REQUIRES DECLARATION: FOREACH_HASH_DECLS;
  */
 
 #define FOREACH_HASH_DECLS \
     Tcl_HashEntry *hPtr;Tcl_HashSearch search
-#define FOREACH_HASH(key,val,tablePtr) \
-    for(hPtr=Tcl_FirstHashEntry((tablePtr),&search); hPtr!=NULL ? \
-	    (*(void **)&(key)=Tcl_GetHashKey((tablePtr),hPtr),\
-	    *(void **)&(val)=Tcl_GetHashValue(hPtr),1):0; hPtr=Tcl_NextHashEntry(&search))
-#define FOREACH_HASH_VALUE(val,tablePtr) \
-    for(hPtr=Tcl_FirstHashEntry((tablePtr),&search); hPtr!=NULL ? \
-	    (*(void **)&(val)=Tcl_GetHashValue(hPtr),1):0;hPtr=Tcl_NextHashEntry(&search))
+#define FOREACH_HASH(key, val, tablePtr) \
+    for(hPtr = Tcl_FirstHashEntry((tablePtr), &search); hPtr != NULL ? \
+	    (*(void **)&(key) = Tcl_GetHashKey((tablePtr), hPtr), \
+	    *(void **)&(val) = Tcl_GetHashValue(hPtr), 1) : 0; \
+	    hPtr = Tcl_NextHashEntry(&search))
+#define FOREACH_HASH_KEY(key, tablePtr) \
+    for(hPtr = Tcl_FirstHashEntry((tablePtr), &search); hPtr != NULL ? \
+	    (*(void **)&(key) = Tcl_GetHashKey((tablePtr), hPtr), 1) : 0; \
+	    hPtr = Tcl_NextHashEntry(&search))
+#define FOREACH_HASH_VALUE(val, tablePtr) \
+    for(hPtr = Tcl_FirstHashEntry((tablePtr), &search); hPtr != NULL ? \
+	    (*(void **)&(val) = Tcl_GetHashValue(hPtr), 1) : 0; \
+	    hPtr = Tcl_NextHashEntry(&search))
 
 /*
  * Convenience macro for duplicating a list. Needs no external declaration,
