@@ -374,7 +374,7 @@ TclOODelMethodRef(
 /*
  * ----------------------------------------------------------------------
  *
- * TclOONewBasicMethod --
+ * TclOODefineBasicMethods --
  *
  *	Helper that makes it cleaner to create very simple methods during
  *	basic system initialization. Not suitable for general use.
@@ -383,17 +383,21 @@ TclOODelMethodRef(
  */
 
 void
-TclOONewBasicMethod(
-    Class *clsPtr,		/* Class to attach the method to. */
-    const DeclaredClassMethod *dcm)
-				/* Name of the method, whether it is public,
-				 * and the function to implement it. */
+TclOODefineBasicMethods(
+    Class *clsPtr,		/* Class to attach the methods to. */
+    const DeclaredClassMethod *dcmAry)
+				/* Static table of method definitions. */
 {
-    Tcl_Obj *namePtr = Tcl_NewStringObj(dcm->name, -1);
+    int i;
 
-    TclNewMethod((Tcl_Class) clsPtr, namePtr,
-	    (dcm->isPublic ? PUBLIC_METHOD : 0), &dcm->definition, NULL);
-    Tcl_BounceRefCount(namePtr);
+    for (i = 0 ; dcmAry[i].name ; i++) {
+	Tcl_Obj *namePtr = Tcl_NewStringObj(dcmAry[i].name, TCL_AUTO_LENGTH);
+
+	TclNewMethod((Tcl_Class) clsPtr, namePtr,
+		(dcmAry[i].isPublic ? PUBLIC_METHOD : 0),
+		&dcmAry[i].definition, NULL);
+	Tcl_BounceRefCount(namePtr);
+    }
 }
 
 /*
@@ -1204,100 +1208,118 @@ RenderDeclarerName(
 	((len) > LIMIT ? LIMIT : (int)(len)), (str), ((len) > LIMIT ? "..." : "")
 
 static void
-MethodErrorHandler(
+CommonMethErrorHandler(
     Tcl_Interp *interp,
-    TCL_UNUSED(Tcl_Obj *) /*methodNameObj*/)
-	// We pull the method name out of context instead of from argument
+    const char *special)
+
 {
-    Tcl_Size nameLen, objectNameLen;
-    CallContext *contextPtr = (CallContext *)
-	    ((Interp *) interp)->varFramePtr->clientData;
+    Tcl_Size objectNameLen;
+    CallContext *contextPtr = (CallContext *)((Interp *) interp)->varFramePtr->clientData;
     Method *mPtr = contextPtr->callPtr->chain[contextPtr->index].mPtr;
-    const char *objectName, *kindName, *methodName =
-	    Tcl_GetStringFromObj(mPtr->namePtr, &nameLen);
-    Object *declarerPtr;
+    const char *objectName, *kindName = "instance";
+    Object *declarerPtr = NULL;
 
     if (mPtr->declaringObjectPtr != NULL) {
 	declarerPtr = mPtr->declaringObjectPtr;
 	kindName = "object";
-    } else {
-	if (mPtr->declaringClassPtr == NULL) {
-	    Tcl_Panic("method not declared in class or object");
-	}
+
+    } else if (mPtr->declaringClassPtr != NULL) {
+
+
 	declarerPtr = mPtr->declaringClassPtr->thisPtr;
 	kindName = "class";
     }
 
-    objectName = Tcl_GetStringFromObj(TclOOObjectName(interp, declarerPtr),
+    if (declarerPtr) {
+	objectName = TclGetStringFromObj(TclOOObjectName(interp, declarerPtr),
 	    &objectNameLen);
-    Tcl_AppendObjToErrorInfo(interp, Tcl_ObjPrintf(
+    } else {
+	objectName = "unknown or deleted";
+	objectNameLen = 18;
+    }
+    if (!special) {
+	Tcl_Size nameLen;
+	const char *methodName = TclGetStringFromObj(mPtr->namePtr, &nameLen);
+	Tcl_AppendObjToErrorInfo(interp, Tcl_ObjPrintf(
 	    "\n    (%s \"%.*s%s\" method \"%.*s%s\" line %d)",
 	    kindName, ELLIPSIFY(objectName, objectNameLen),
 	    ELLIPSIFY(methodName, nameLen), Tcl_GetErrorLine(interp)));
+    } else {
+	Tcl_AppendObjToErrorInfo(interp, Tcl_ObjPrintf(
+	    "\n    (%s \"%.*s%s\" %s line %d)", kindName,
+	    ELLIPSIFY(objectName, objectNameLen), special, Tcl_GetErrorLine(interp)));
+    }
+}
+
+static void
+MethodErrorHandler(
+    Tcl_Interp *interp,
+    TCL_UNUSED(Tcl_Obj *) /*methodNameObj*/)
+	/* We pull the method name out of context instead of from argument */
+{
+    CommonMethErrorHandler(interp, NULL);
 }
 
 static void
 ConstructorErrorHandler(
     Tcl_Interp *interp,
     TCL_UNUSED(Tcl_Obj *) /*methodNameObj*/)
-	// Ignore. We know it is the constructor.
+	/* Ignore. We know it is the constructor. */
 {
-    CallContext *contextPtr = (CallContext *)
-	    ((Interp *) interp)->varFramePtr->clientData;
-    Method *mPtr = contextPtr->callPtr->chain[contextPtr->index].mPtr;
-    Object *declarerPtr;
-    const char *objectName, *kindName;
-    Tcl_Size objectNameLen;
 
-    if (mPtr->declaringObjectPtr != NULL) {
-	declarerPtr = mPtr->declaringObjectPtr;
-	kindName = "object";
-    } else {
-	if (mPtr->declaringClassPtr == NULL) {
-	    Tcl_Panic("method not declared in class or object");
-	}
-	declarerPtr = mPtr->declaringClassPtr->thisPtr;
-	kindName = "class";
-    }
 
-    objectName = Tcl_GetStringFromObj(TclOOObjectName(interp, declarerPtr),
-	    &objectNameLen);
-    Tcl_AppendObjToErrorInfo(interp, Tcl_ObjPrintf(
-	    "\n    (%s \"%.*s%s\" constructor line %d)", kindName,
-	    ELLIPSIFY(objectName, objectNameLen), Tcl_GetErrorLine(interp)));
+
+
+
+    CommonMethErrorHandler(interp, "constructor");
+
+
+
+
+
+
 }
 
+
+
+
+
+
+
+
+
+
+
 static void
 DestructorErrorHandler(
     Tcl_Interp *interp,
     TCL_UNUSED(Tcl_Obj *) /*methodNameObj*/)
-	// Ignore. We know it is the destructor.
+	/* Ignore. We know it is the destructor. */
 {
-    CallContext *contextPtr = (CallContext *)
-	    ((Interp *) interp)->varFramePtr->clientData;
-    Method *mPtr = contextPtr->callPtr->chain[contextPtr->index].mPtr;
-    Object *declarerPtr;
-    const char *objectName, *kindName;
-    Tcl_Size objectNameLen;
 
-    if (mPtr->declaringObjectPtr != NULL) {
-	declarerPtr = mPtr->declaringObjectPtr;
-	kindName = "object";
-    } else {
-	if (mPtr->declaringClassPtr == NULL) {
-	    Tcl_Panic("method not declared in class or object");
-	}
-	declarerPtr = mPtr->declaringClassPtr->thisPtr;
-	kindName = "class";
-    }
 
-    objectName = Tcl_GetStringFromObj(TclOOObjectName(interp, declarerPtr),
-	    &objectNameLen);
-    Tcl_AppendObjToErrorInfo(interp, Tcl_ObjPrintf(
-	    "\n    (%s \"%.*s%s\" destructor line %d)", kindName,
-	    ELLIPSIFY(objectName, objectNameLen), Tcl_GetErrorLine(interp)));
+
+
+
+    CommonMethErrorHandler(interp, "destructor");
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 }
-
+
 /*
  * ----------------------------------------------------------------------
  *
@@ -1353,7 +1375,7 @@ CloneProcedureMethod(
 
 	    TclNewObj(argObj);
 	    Tcl_ListObjAppendElement(NULL, argObj,
-		    Tcl_NewStringObj(localPtr->name, -1));
+		    Tcl_NewStringObj(localPtr->name, TCL_AUTO_LENGTH));
 	    if (localPtr->defValuePtr != NULL) {
 		Tcl_ListObjAppendElement(NULL, argObj, localPtr->defValuePtr);
 	    }
@@ -1426,7 +1448,7 @@ TclOONewForwardInstanceMethod(
     }
     if (prefixLen < 1) {
 	Tcl_SetObjResult(interp, Tcl_NewStringObj(
-		"method forward prefix must be non-empty", -1));
+		"method forward prefix must be non-empty", TCL_AUTO_LENGTH));
 	Tcl_SetErrorCode(interp, "TCL", "OO", "BAD_FORWARD", (char *)NULL);
 	return NULL;
     }
@@ -1465,7 +1487,7 @@ TclOONewForwardMethod(
     }
     if (prefixLen < 1) {
 	Tcl_SetObjResult(interp, Tcl_NewStringObj(
-		"method forward prefix must be non-empty", -1));
+		"method forward prefix must be non-empty", TCL_AUTO_LENGTH));
 	Tcl_SetErrorCode(interp, "TCL", "OO", "BAD_FORWARD", (char *)NULL);
 	return NULL;
     }
