@@ -5,9 +5,9 @@
  *	applications will probably call Tcl_SetPanicProc() to set an
  *	application-specific panic procedure.
  *
- * Copyright (c) 1988-1993 The Regents of the University of California.
- * Copyright (c) 1994 Sun Microsystems, Inc.
- * Copyright (c) 1998-1999 by Scriptics Corporation.
+ * Copyright © 1988-1993 The Regents of the University of California.
+ * Copyright © 1994 Sun Microsystems, Inc.
+ * Copyright © 1998-1999 Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -15,7 +15,7 @@
 
 #include "tclInt.h"
 #if defined(_WIN32) || defined(__CYGWIN__)
-    MODULE_SCOPE TCL_NORETURN void tclWinDebugPanic(const char *format, ...);
+    MODULE_SCOPE void tclWinDebugPanic(const char *format, ...);
 #endif
 
 /*
@@ -23,11 +23,7 @@
  * procedure.
  */
 
-#if defined(__CYGWIN__)
-static TCL_NORETURN Tcl_PanicProc *panicProc = tclWinDebugPanic;
-#else
-static TCL_NORETURN1 Tcl_PanicProc *panicProc = NULL;
-#endif
+static Tcl_PanicProc *panicProc = NULL;
 
 /*
  *----------------------------------------------------------------------
@@ -45,84 +41,12 @@ static TCL_NORETURN1 Tcl_PanicProc *panicProc = NULL;
  *----------------------------------------------------------------------
  */
 
-void
+const char *
 Tcl_SetPanicProc(
-    TCL_NORETURN1 Tcl_PanicProc *proc)
+    Tcl_PanicProc *proc)
 {
-#if defined(_WIN32)
-    /* tclWinDebugPanic only installs if there is no panicProc yet. */
-    if ((proc != tclWinDebugPanic) || (panicProc == NULL))
-#elif defined(__CYGWIN__)
-    if (proc == NULL)
-	panicProc = tclWinDebugPanic;
-    else
-#endif
     panicProc = proc;
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * Tcl_PanicVA --
- *
- *	Print an error message and kill the process.
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	The process dies, entering the debugger if possible.
- *
- *----------------------------------------------------------------------
- */
-
-void
-Tcl_PanicVA(
-    const char *format,		/* Format string, suitable for passing to
-				 * fprintf. */
-    va_list argList)		/* Variable argument list. */
-{
-    char *arg1, *arg2, *arg3;	/* Additional arguments (variable in number)
-				 * to pass to fprintf. */
-    char *arg4, *arg5, *arg6, *arg7, *arg8;
-
-    arg1 = va_arg(argList, char *);
-    arg2 = va_arg(argList, char *);
-    arg3 = va_arg(argList, char *);
-    arg4 = va_arg(argList, char *);
-    arg5 = va_arg(argList, char *);
-    arg6 = va_arg(argList, char *);
-    arg7 = va_arg(argList, char *);
-    arg8 = va_arg(argList, char *);
-
-    if (panicProc != NULL) {
-	panicProc(format, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8);
-#ifdef _WIN32
-    } else if (IsDebuggerPresent()) {
-	tclWinDebugPanic(format, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8);
-#endif
-    } else {
-	fprintf(stderr, format, arg1, arg2, arg3, arg4, arg5, arg6, arg7,
-		arg8);
-	fprintf(stderr, "\n");
-	fflush(stderr);
-#if defined(_WIN32) || defined(__CYGWIN__)
-#   if defined(__GNUC__)
-	__builtin_trap();
-#   elif defined(_WIN64)
-	__debugbreak();
-#   elif defined(_MSC_VER) && defined (_M_IX86)
-	_asm {int 3}
-#   else
-	DebugBreak();
-#   endif
-#endif
-#if defined(_WIN32)
-	ExitProcess(1);
-#else
-	abort();
-#endif
-    }
+    return Tcl_InitSubsystems();
 }
 
 /*
@@ -141,24 +65,59 @@ Tcl_PanicVA(
  *----------------------------------------------------------------------
  */
 
-/* ARGSUSED */
-
 /*
  * The following comment is here so that Coverity's static analyzer knows that
  * a Tcl_Panic() call can never return and avoids lots of false positives.
  */
 
 /* coverity[+kill] */
-void
+TCL_NORETURN void
 Tcl_Panic(
     const char *format,
     ...)
 {
     va_list argList;
+    char *arg1, *arg2, *arg3;	/* Additional arguments (variable in number)
+				 * to pass to fprintf. */
+    char *arg4, *arg5, *arg6, *arg7, *arg8;
 
     va_start(argList, format);
-    Tcl_PanicVA(format, argList);
+    arg1 = va_arg(argList, char *);
+    arg2 = va_arg(argList, char *);
+    arg3 = va_arg(argList, char *);
+    arg4 = va_arg(argList, char *);
+    arg5 = va_arg(argList, char *);
+    arg6 = va_arg(argList, char *);
+    arg7 = va_arg(argList, char *);
+    arg8 = va_arg(argList, char *);
     va_end (argList);
+
+    if (panicProc != NULL) {
+	panicProc(format, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8);
+    } else {
+#if defined(_WIN32) || defined(__CYGWIN__)
+    tclWinDebugPanic(format, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8);
+#else
+	fprintf(stderr, format, arg1, arg2, arg3, arg4, arg5, arg6, arg7,
+		arg8);
+	fprintf(stderr, "\n");
+	fflush(stderr);
+#endif
+    }
+#if defined(__GNUC__)
+    __builtin_trap();
+#elif defined(_WIN64)
+    __debugbreak();
+#elif defined(_MSC_VER) && defined (_M_IX86)
+    _asm {int 3}
+#elif defined(_WIN32)
+    DebugBreak();
+#endif
+#if defined(_WIN32)
+    ExitProcess(1);
+#else
+    abort();
+#endif
 }
 
 /*
