@@ -5,8 +5,8 @@
  *	sure that widget records and other data structures aren't reallocated
  *	when there are nested functions that depend on their existence.
  *
- * Copyright (c) 1991-1994 The Regents of the University of California.
- * Copyright (c) 1994-1998 Sun Microsystems, Inc.
+ * Copyright © 1991-1994 The Regents of the University of California.
+ * Copyright © 1994-1998 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -21,8 +21,8 @@
  */
 
 typedef struct {
-    ClientData clientData;	/* Address of preserved block. */
-    int refCount;		/* Number of Tcl_Preserve calls in effect for
+    void *clientData;		/* Address of preserved block. */
+    size_t refCount;		/* Number of Tcl_Preserve calls in effect for
 				 * block. */
     int mustFree;		/* Non-zero means Tcl_EventuallyFree was
 				 * called while a Tcl_Preserve call was in
@@ -36,10 +36,11 @@ typedef struct {
  * These variables are protected by "preserveMutex".
  */
 
-static Reference *refArray = NULL;	/* First in array of references. */
-static int spaceAvl = 0;	/* Total number of structures available at
+static Reference *refArray = NULL;
+				/* First in array of references. */
+static size_t spaceAvl = 0;	/* Total number of structures available at
 				 * *firstRefPtr. */
-static int inUse = 0;		/* Count of structures currently in use in
+static size_t inUse = 0;	/* Count of structures currently in use in
 				 * refArray. */
 TCL_DECLARE_MUTEX(preserveMutex)/* To protect the above statics */
 
@@ -53,7 +54,7 @@ TCL_DECLARE_MUTEX(preserveMutex)/* To protect the above statics */
  * objects that we don't want to live any longer than necessary.
  */
 
-typedef struct HandleStruct {
+typedef struct {
     void *ptr;			/* Pointer to the memory block being tracked.
 				 * This field will become NULL when the memory
 				 * block is deleted. This field must be the
@@ -63,7 +64,7 @@ typedef struct HandleStruct {
 				 * ensure that the contents of the handle are
 				 * not changed by anyone else. */
 #endif
-    int refCount;		/* Number of TclHandlePreserve() calls in
+    size_t refCount;		/* Number of TclHandlePreserve() calls in
 				 * effect on this handle. */
 } HandleStruct;
 
@@ -83,13 +84,12 @@ typedef struct HandleStruct {
  *----------------------------------------------------------------------
  */
 
-	/* ARGSUSED */
 void
 TclFinalizePreserve(void)
 {
     Tcl_MutexLock(&preserveMutex);
     if (spaceAvl != 0) {
-	ckfree(refArray);
+	Tcl_Free(refArray);
 	refArray = NULL;
 	inUse = 0;
 	spaceAvl = 0;
@@ -118,10 +118,10 @@ TclFinalizePreserve(void)
 
 void
 Tcl_Preserve(
-    ClientData clientData)	/* Pointer to malloc'ed block of memory. */
+    void *clientData)		/* Pointer to malloc'ed block of memory. */
 {
     Reference *refPtr;
-    int i;
+    size_t i;
 
     /*
      * See if there is already a reference for this pointer. If so, just
@@ -144,7 +144,7 @@ Tcl_Preserve(
 
     if (inUse == spaceAvl) {
 	spaceAvl = spaceAvl ? 2*spaceAvl : INITIAL_SIZE;
-	refArray = ckrealloc(refArray, spaceAvl * sizeof(Reference));
+	refArray = (Reference *)Tcl_Realloc(refArray, spaceAvl * sizeof(Reference));
     }
 
     /*
@@ -155,7 +155,7 @@ Tcl_Preserve(
     refPtr->clientData = clientData;
     refPtr->refCount = 1;
     refPtr->mustFree = 0;
-    refPtr->freeProc = TCL_STATIC;
+    refPtr->freeProc = 0;
     inUse += 1;
     Tcl_MutexUnlock(&preserveMutex);
 }
@@ -181,10 +181,10 @@ Tcl_Preserve(
 
 void
 Tcl_Release(
-    ClientData clientData)	/* Pointer to malloc'ed block of memory. */
+    void *clientData)		/* Pointer to malloc'ed block of memory. */
 {
     Reference *refPtr;
-    int i;
+    size_t i;
 
     Tcl_MutexLock(&preserveMutex);
     for (i=0, refPtr=refArray ; i<inUse ; i++, refPtr++) {
@@ -195,7 +195,7 @@ Tcl_Release(
 	    continue;
 	}
 
-	if (--refPtr->refCount != 0) {
+	if (refPtr->refCount-- > 1) {
 	    Tcl_MutexUnlock(&preserveMutex);
 	    return;
 	}
@@ -224,9 +224,9 @@ Tcl_Release(
 	Tcl_MutexUnlock(&preserveMutex);
 	if (mustFree) {
 	    if (freeProc == TCL_DYNAMIC) {
-		ckfree(clientData);
+		Tcl_Free(clientData);
 	    } else {
-		freeProc(clientData);
+		freeProc((char *)clientData);
 	    }
 	}
 	return;
@@ -253,18 +253,18 @@ Tcl_Release(
  *	None.
  *
  * Side effects:
- *	Ptr may be released by calling ckfree().
+ *	Ptr may be released by calling free().
  *
  *----------------------------------------------------------------------
  */
 
 void
 Tcl_EventuallyFree(
-    ClientData clientData,	/* Pointer to malloc'ed block of memory. */
+    void *clientData,		/* Pointer to malloc'ed block of memory. */
     Tcl_FreeProc *freeProc)	/* Function to actually do free. */
 {
     Reference *refPtr;
-    int i;
+    size_t i;
 
     /*
      * See if there is a reference for this pointer. If so, set its "mustFree"
@@ -291,7 +291,7 @@ Tcl_EventuallyFree(
      */
 
     if (freeProc == TCL_DYNAMIC) {
-	ckfree(clientData);
+	Tcl_Free(clientData);
     } else {
 	freeProc(clientData);
     }
@@ -327,7 +327,7 @@ TclHandleCreate(
 				 * be tracked for deletion. Must not be
 				 * NULL. */
 {
-    HandleStruct *handlePtr = ckalloc(sizeof(HandleStruct));
+    HandleStruct *handlePtr = (HandleStruct *)Tcl_Alloc(sizeof(HandleStruct));
 
     handlePtr->ptr = ptr;
 #ifdef TCL_MEM_DEBUG
@@ -377,7 +377,7 @@ TclHandleFree(
 #endif
     handlePtr->ptr = NULL;
     if (handlePtr->refCount == 0) {
-	ckfree(handlePtr);
+	Tcl_Free(handlePtr);
     }
 }
 
@@ -459,8 +459,8 @@ TclHandleRelease(
 		handlePtr, handlePtr->ptr2, handlePtr->ptr);
     }
 #endif
-    if ((--handlePtr->refCount == 0) && (handlePtr->ptr == NULL)) {
-	ckfree(handlePtr);
+    if ((handlePtr->refCount-- <= 1) && (handlePtr->ptr == NULL)) {
+	Tcl_Free(handlePtr);
     }
 }
 
