@@ -4,8 +4,8 @@
  *	Stub object that will be statically linked into extensions that want
  *	to access Tcl.
  *
- * Copyright (c) 1998-1999 Scriptics Corporation.
- * Copyright (c) 1998 Paul Duffin.
+ * Copyright © 1998-1999 Scriptics Corporation.
+ * Copyright © 1998 Paul Duffin.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -17,11 +17,13 @@ MODULE_SCOPE const TclStubs *tclStubsPtr;
 MODULE_SCOPE const TclPlatStubs *tclPlatStubsPtr;
 MODULE_SCOPE const TclIntStubs *tclIntStubsPtr;
 MODULE_SCOPE const TclIntPlatStubs *tclIntPlatStubsPtr;
+MODULE_SCOPE void *tclStubsHandle;
 
 const TclStubs *tclStubsPtr = NULL;
 const TclPlatStubs *tclPlatStubsPtr = NULL;
 const TclIntStubs *tclIntStubsPtr = NULL;
 const TclIntPlatStubs *tclIntPlatStubsPtr = NULL;
+void *tclStubsHandle = NULL;
 
 /*
  * Use our own ISDIGIT to avoid linking to libc on windows
@@ -51,42 +53,31 @@ MODULE_SCOPE const char *
 Tcl_InitStubs(
     Tcl_Interp *interp,
     const char *version,
-    int exact)
+    int exact,
+    int magic)
 {
     Interp *iPtr = (Interp *)interp;
     const char *actualVersion = NULL;
     void *pkgData = NULL;
     const TclStubs *stubsPtr = iPtr->stubTable;
+    const char *tclName = (((exact&0xFF00) >= 0x900) ? "tcl" : "Tcl");
 
+#undef TCL_STUB_MAGIC /* We need the TCL_STUB_MAGIC from Tcl 8.x here */
+#define TCL_STUB_MAGIC		((int) 0xFCA3BACF)
     /*
      * We can't optimize this check by caching tclStubsPtr because that
      * prevents apps from being able to load/unload Tcl dynamically multiple
      * times. [Bug 615304]
      */
 
-    if (!stubsPtr || (stubsPtr->magic != TCL_STUB_MAGIC)) {
-	exact &= 0xFFFF00; /* Filter out minor/major Tcl version */
-	if (!exact) {
-	    exact = 0x060800;
-	}
-	if (stubsPtr && (stubsPtr->magic == ((int)0xFCA3BACB + (int)sizeof(void *)))
-		&& ((exact|0x010000) == 0x070800)) {
-	    /* We are running in Tcl 9.x, but extension is compiled with 8.6 */
-	    stubsPtr->tcl_SetObjResult(interp, stubsPtr->tcl_ObjPrintf(
-		    "this extension is compiled for Tcl %d.%d",
-		    (exact & 0x0FF00)>>8, (exact & 0x0FF0000)>>16));
-	} else {
-	    iPtr->result = (char *)"interpreter uses an incompatible stubs mechanism";
-	    iPtr->freeProc = TCL_STATIC;
-	}
+    if (!stubsPtr || (stubsPtr->magic != (((exact&0xFF00) >= 0x900) ? magic : TCL_STUB_MAGIC))) {
+	iPtr->legacyResult = "interpreter uses an incompatible stubs mechanism";
+	iPtr->legacyFreeProc = 0; /* TCL_STATIC */
 	return NULL;
     }
 
-    actualVersion = stubsPtr->tcl_PkgRequireEx(interp, "Tcl", version, 0, &pkgData);
+    actualVersion = stubsPtr->tcl_PkgRequireEx(interp, tclName, version, 0, &pkgData);
     if (actualVersion == NULL) {
-	/* Even when the Tcl version does not match, the caller should at least be
-	 * able to use Tcl_GetObjResult/Tcl_GetString/Tcl_Panic for error-handling */
-	tclStubsPtr = stubsPtr; /* See: [fd8341e496] */
 	return NULL;
     }
     if (exact&1) {
@@ -105,22 +96,29 @@ Tcl_InitStubs(
 	    }
 	    if (*p || ISDIGIT(*q)) {
 		/* Construct error message */
-		stubsPtr->tcl_PkgRequireEx(interp, "Tcl", version, 1, NULL);
+		stubsPtr->tcl_PkgRequireEx(interp, tclName, version, 1, NULL);
 		return NULL;
 	    }
 	} else {
-	    actualVersion = stubsPtr->tcl_PkgRequireEx(interp, "Tcl", version, 1, NULL);
+	    actualVersion = stubsPtr->tcl_PkgRequireEx(interp, tclName, version, 1, NULL);
 	    if (actualVersion == NULL) {
 		return NULL;
 	    }
 	}
     }
-    tclStubsPtr = (TclStubs *)pkgData;
+    if (((exact&0xFF00) < 0x900)) {
+	/* We are running Tcl 8.x */
+	stubsPtr = (TclStubs *)pkgData;
+    }
+    if (tclStubsHandle == NULL) {
+	tclStubsHandle = INT2PTR(-1);
+    }
+    tclStubsPtr = stubsPtr;
 
-    if (tclStubsPtr->hooks) {
-	tclPlatStubsPtr = tclStubsPtr->hooks->tclPlatStubs;
-	tclIntStubsPtr = tclStubsPtr->hooks->tclIntStubs;
-	tclIntPlatStubsPtr = tclStubsPtr->hooks->tclIntPlatStubs;
+    if (stubsPtr->hooks) {
+	tclPlatStubsPtr = stubsPtr->hooks->tclPlatStubs;
+	tclIntStubsPtr = stubsPtr->hooks->tclIntStubs;
+	tclIntPlatStubsPtr = stubsPtr->hooks->tclIntPlatStubs;
     } else {
 	tclPlatStubsPtr = NULL;
 	tclIntStubsPtr = NULL;
