@@ -2,7 +2,8 @@ if {[info sharedlibextension] != ".dll"} return
 if {[package vsatisfies [package provide Tcl] 9.0-]} {
     package ifneeded dde 1.4.5 \
 	    [list load [file join $dir tcl9dde14.dll] Dde]
-} elseif {[::tcl::pkgconfig get debug]} {
+} elseif {![package vsatisfies [package provide Tcl] 8.7]
+	&& [::tcl::pkgconfig get debug]} {
     package ifneeded dde 1.4.5 \
 	    [list load [file join $dir tcldde14g.dll] Dde]
 } else {
