@@ -44,8 +44,8 @@ mp_err mp_div_d(const mp_int *a, mp_digit b, mp_int *c, mp_digit *d)
    }
 
    /* three? */
-   if (MP_HAS(MP_DIV_3) && (b == 3u)) {
-      return mp_div_3(a, c, d);
+   if (MP_HAS(S_MP_DIV_3) && (b == 3u)) {
+      return s_mp_div_3(a, c, d);
    }
 
    /* no easy answer [c'est la vie].  Just division */
