@@ -4,7 +4,7 @@
  *	This procedure provides a version of the TclLoadFile that works with
  *	the "dlopen" and "dlsym" library procedures for dynamic loading.
  *
- * Copyright (c) 1995-1997 Sun Microsystems, Inc.
+ * Copyright © 1995-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -108,7 +108,11 @@ TclpDlopen(
 	Tcl_DString ds;
 	const char *fileName = TclGetString(pathPtr);
 
-	native = Tcl_UtfToExternalDString(NULL, fileName, -1, &ds);
+	if (Tcl_UtfToExternalDStringEx(interp, NULL, fileName, TCL_INDEX_NONE, 0, &ds, NULL) != TCL_OK) {
+	    Tcl_DStringFree(&ds);
+	    return TCL_ERROR;
+	}
+	native = Tcl_DStringValue(&ds);
 	/*
 	 * Use (RTLD_NOW|RTLD_LOCAL) as default, see [Bug #3216070]
 	 */
@@ -131,7 +135,7 @@ TclpDlopen(
 	}
 	return TCL_ERROR;
     }
-    newHandle = (Tcl_LoadHandle)ckalloc(sizeof(*newHandle));
+    newHandle = (Tcl_LoadHandle)Tcl_Alloc(sizeof(*newHandle));
     newHandle->clientData = handle;
     newHandle->findSymbolProcPtr = &FindSymbol;
     newHandle->unloadFileProcPtr = &UnloadFile;
@@ -179,15 +183,44 @@ FindSymbol(
      * the underscore.
      */
 
-    native = Tcl_UtfToExternalDString(NULL, symbol, -1, &ds);
+    if (Tcl_UtfToExternalDStringEx(interp, NULL, symbol, TCL_INDEX_NONE, 0, &ds, NULL) != TCL_OK) {
+	Tcl_DStringFree(&ds);
+	return NULL;
+    }
+    native = Tcl_DStringValue(&ds);
     proc = dlsym(handle, native);	/* INTL: Native. */
     if (proc == NULL) {
 	Tcl_DStringInit(&newName);
 	TclDStringAppendLiteral(&newName, "_");
-	native = Tcl_DStringAppend(&newName, native, -1);
+	native = Tcl_DStringAppend(&newName, native, TCL_INDEX_NONE);
 	proc = dlsym(handle, native);	/* INTL: Native. */
 	Tcl_DStringFree(&newName);
     }
+#ifdef __cplusplus
+    if (proc == NULL) {
+	char buf[32];
+	snprintf(buf, sizeof(buf), "%d", (int)Tcl_DStringLength(&ds));
+	Tcl_DStringInit(&newName);
+	TclDStringAppendLiteral(&newName, "__Z");
+	Tcl_DStringAppend(&newName, buf, TCL_INDEX_NONE);
+	Tcl_DStringAppend(&newName, Tcl_DStringValue(&ds), TCL_INDEX_NONE);
+	TclDStringAppendLiteral(&newName, "P10Tcl_Interp");
+	native = Tcl_DStringValue(&newName);
+	proc = dlsym(handle, native + 1);	/* INTL: Native. */
+	if (proc == NULL) {
+	    proc = dlsym(handle, native);	/* INTL: Native. */
+	}
+	if (proc == NULL) {
+	    TclDStringAppendLiteral(&newName, "i");
+	    native = Tcl_DStringValue(&newName);
+	    proc = dlsym(handle, native + 1);	/* INTL: Native. */
+	}
+	if (proc == NULL) {
+	    proc = dlsym(handle, native);	/* INTL: Native. */
+	}
+	Tcl_DStringFree(&newName);
+    }
+#endif
     Tcl_DStringFree(&ds);
     if (proc == NULL) {
 	const char *errorStr = dlerror();
@@ -231,71 +264,9 @@ UnloadFile(
     void *handle = loadHandle->clientData;
 
     dlclose(handle);
-    ckfree(loadHandle);
+    Tcl_Free(loadHandle);
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TclGuessPackageName --
- *
- *	If the "load" command is invoked without providing a package name,
- *	this procedure is invoked to try to figure it out.
- *
- * Results:
- *	Always returns 0 to indicate that we couldn't figure out a package
- *	name; generic code will then try to guess the package from the file
- *	name. A return value of 1 would have meant that we figured out the
- *	package name and put it in bufPtr.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-int
-TclGuessPackageName(
-    const char *fileName,	/* Name of file containing package (already
-				 * translated to local form if needed). */
-    Tcl_DString *bufPtr)	/* Initialized empty dstring. Append package
-				 * name to this if possible. */
-{
-    return 0;
-}
-
-/*
- * These functions are fallbacks if we somehow determine that the platform can
- * do loading from memory but the user wishes to disable it. They just report
- * (gracefully) that they fail.
- */
-
-#ifdef TCL_LOAD_FROM_MEMORY
-
-MODULE_SCOPE void *
-TclpLoadMemoryGetBuffer(
-     size_t size)		/* Dummy: unused by this implementation */
-{
-    return NULL;
-}
-
-MODULE_SCOPE int
-TclpLoadMemory(
-    void *buffer,		/* Dummy: unused by this implementation */
-    size_t size,		/* Dummy: unused by this implementation */
-    int codeSize,		/* Dummy: unused by this implementation */
-    const char *path,	/* Dummy: unused by this implementation */
-    Tcl_LoadHandle *loadHandle,	/* Dummy: unused by this implementation */
-    Tcl_FSUnloadFileProc **unloadProcPtr,
-				/* Dummy: unused by this implementation */
-    int flags)
-				/* Dummy: unused by this implementation */
-{
-    return TCL_ERROR;
-}
-
-#endif /* TCL_LOAD_FROM_MEMORY */
-
 /*
  * Local Variables:
  * mode: c
