@@ -19,35 +19,13 @@
 #   define MODULE_SCOPE extern
 #endif
 
-/*
- * Use preferred dlfcn API on 10.4 and later
- */
-
-#ifndef TCL_DYLD_USE_DLFCN
-#   ifdef NO_DLFCN_H
-#	define TCL_DYLD_USE_DLFCN 0
-#   else
-#	define TCL_DYLD_USE_DLFCN 1
-#   endif
-#endif
-
-/*
- * Use deprecated NSModule API only to support 10.3 and earlier:
- */
-
-#ifndef TCL_DYLD_USE_NSMODULE
-#   define TCL_DYLD_USE_NSMODULE 0
-#endif
-
 /*
  * Use includes for the API we're using.
  */
 
-#if TCL_DYLD_USE_DLFCN
-#   include <dlfcn.h>
-#endif /* TCL_DYLD_USE_DLFCN */
+#include <dlfcn.h>
 
-#if TCL_DYLD_USE_NSMODULE || defined(TCL_LOAD_FROM_MEMORY)
+#if defined(TCL_LOAD_FROM_MEMORY)
 #if defined (__clang__) || ((__GNUC__)  && ((__GNUC__ > 4) || ((__GNUC__ == 4) && (__GNUC_MINOR__ > 5))))
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 #endif
@@ -62,20 +40,16 @@ typedef struct Tcl_DyldModuleHandle {
     struct Tcl_DyldModuleHandle *nextPtr;
     NSModule module;
 } Tcl_DyldModuleHandle;
-#endif /* TCL_DYLD_USE_NSMODULE || TCL_LOAD_FROM_MEMORY */
+#endif /* TCL_LOAD_FROM_MEMORY */
 
 typedef struct {
     void *dlHandle;
-#if TCL_DYLD_USE_NSMODULE || defined(TCL_LOAD_FROM_MEMORY)
+#if defined(TCL_LOAD_FROM_MEMORY)
     const struct mach_header *dyldLibHeader;
     Tcl_DyldModuleHandle *modulePtr;
 #endif
 } Tcl_DyldLoadHandle;
 
-#if TCL_DYLD_USE_DLFCN || defined(TCL_LOAD_FROM_MEMORY)
-MODULE_SCOPE long	tclMacOSXDarwinRelease;
-#endif
-
 /*
  * Static functions defined in this file.
  */
@@ -101,7 +75,7 @@ static void		UnloadFile(Tcl_LoadHandle handle);
  *----------------------------------------------------------------------
  */
 
-#if TCL_DYLD_USE_NSMODULE || defined(TCL_LOAD_FROM_MEMORY)
+#if defined(TCL_LOAD_FROM_MEMORY)
 static const char *
 DyldOFIErrorMsg(
     int err)
@@ -123,7 +97,7 @@ DyldOFIErrorMsg(
 	return "unknown error";
     }
 }
-#endif /* TCL_DYLD_USE_NSMODULE || TCL_LOAD_FROM_MEMORY */
+#endif /* TCL_LOAD_FROM_MEMORY */
 
 /*
  *----------------------------------------------------------------------
@@ -160,22 +134,15 @@ TclpDlopen(
     Tcl_DyldLoadHandle *dyldLoadHandle;
     Tcl_LoadHandle newHandle;
     void *dlHandle = NULL;
-#if TCL_DYLD_USE_NSMODULE || defined(TCL_LOAD_FROM_MEMORY)
+#if defined(TCL_LOAD_FROM_MEMORY)
     const struct mach_header *dyldLibHeader = NULL;
     Tcl_DyldModuleHandle *modulePtr = NULL;
 #endif
-#if TCL_DYLD_USE_NSMODULE
-    NSLinkEditErrors editError;
-    int errorNumber;
-    const char *errorName, *objFileImageErrMsg = NULL;
-#endif /* TCL_DYLD_USE_NSMODULE */
     const char *errMsg = NULL;
     int result;
     Tcl_DString ds;
     const char *nativePath, *nativeFileName = NULL;
-#if TCL_DYLD_USE_DLFCN
     int dlopenflags = 0;
-#endif /* TCL_DYLD_USE_DLFCN */
 
     /*
      * First try the full path the user gave us. This is particularly
@@ -191,7 +158,6 @@ TclpDlopen(
     }
     nativeFileName = Tcl_DStringValue(&ds);
 
-#if TCL_DYLD_USE_DLFCN
     /*
      * Use (RTLD_NOW|RTLD_LOCAL) as default, see [Bug #3216070]
      */
@@ -219,79 +185,14 @@ TclpDlopen(
 	    errMsg = dlerror();
 	}
     }
-#endif /* TCL_DYLD_USE_DLFCN */
 
-    if (!dlHandle) {
-#if TCL_DYLD_USE_NSMODULE
-	dyldLibHeader = NSAddImage(nativePath,
-		NSADDIMAGE_OPTION_RETURN_ON_ERROR);
-	if (!dyldLibHeader) {
-	    NSLinkEditError(&editError, &errorNumber, &errorName, &errMsg);
-	    if (editError == NSLinkEditFileAccessError) {
-		/*
-		 * The requested file was not found. Let the OS loader examine
-		 * the binary search path for whatever string the user gave us
-		 * which hopefully refers to a file on the binary path.
-		 */
-
-		dyldLibHeader = NSAddImage(nativeFileName,
-			NSADDIMAGE_OPTION_WITH_SEARCHING |
-			NSADDIMAGE_OPTION_RETURN_ON_ERROR);
-		if (!dyldLibHeader) {
-		    NSLinkEditError(&editError, &errorNumber, &errorName,
-			    &errMsg);
-		}
-	    } else if ((editError == NSLinkEditFileFormatError
-		    && errorNumber == EBADMACHO)
-		    || editError == NSLinkEditOtherError){
-		NSObjectFileImageReturnCode err;
-		NSObjectFileImage dyldObjFileImage;
-		NSModule module;
-
-		/*
-		 * The requested file was found but was not of type MH_DYLIB,
-		 * attempt to load it as a MH_BUNDLE.
-		 */
-
-		err = NSCreateObjectFileImageFromFile(nativePath,
-			&dyldObjFileImage);
-		if (err == NSObjectFileImageSuccess && dyldObjFileImage) {
-		    int nsflags = NSLINKMODULE_OPTION_RETURN_ON_ERROR;
-		    if (!(flags & 1)) {
-			nsflags |= NSLINKMODULE_OPTION_PRIVATE;
-		    }
-		    if (!(flags & 2)) {
-			nsflags |= NSLINKMODULE_OPTION_BINDNOW;
-		    }
-		    module = NSLinkModule(dyldObjFileImage, nativePath, nsflags);
-		    NSDestroyObjectFileImage(dyldObjFileImage);
-		    if (module) {
-			modulePtr = (Tcl_DyldModuleHandle *)Tcl_Alloc(sizeof(Tcl_DyldModuleHandle));
-			modulePtr->module = module;
-			modulePtr->nextPtr = NULL;
-		    } else {
-			NSLinkEditError(&editError, &errorNumber, &errorName,
-				&errMsg);
-		    }
-		} else {
-		    objFileImageErrMsg = DyldOFIErrorMsg(err);
-		}
-	    }
-	}
-#endif /* TCL_DYLD_USE_NSMODULE */
-    }
-
-    if (dlHandle
-#if TCL_DYLD_USE_NSMODULE
-	    || dyldLibHeader || modulePtr
-#endif /* TCL_DYLD_USE_NSMODULE */
-    ) {
+    if (dlHandle) {
 	dyldLoadHandle = (Tcl_DyldLoadHandle *)Tcl_Alloc(sizeof(Tcl_DyldLoadHandle));
 	dyldLoadHandle->dlHandle = dlHandle;
-#if TCL_DYLD_USE_NSMODULE || defined(TCL_LOAD_FROM_MEMORY)
+#if defined(TCL_LOAD_FROM_MEMORY)
 	dyldLoadHandle->dyldLibHeader = dyldLibHeader;
 	dyldLoadHandle->modulePtr = modulePtr;
-#endif /* TCL_DYLD_USE_NSMODULE || TCL_LOAD_FROM_MEMORY */
+#endif /* TCL_LOAD_FROM_MEMORY */
 	newHandle = (Tcl_LoadHandle)Tcl_Alloc(sizeof(*newHandle));
 	newHandle->clientData = dyldLoadHandle;
 	newHandle->findSymbolProcPtr = &FindSymbol;
@@ -306,13 +207,6 @@ TclpDlopen(
 	if (errMsg != NULL) {
 	    Tcl_AppendToObj(errObj, errMsg, TCL_INDEX_NONE);
 	}
-#if TCL_DYLD_USE_NSMODULE
-	if (objFileImageErrMsg) {
-	    Tcl_AppendPrintfToObj(errObj,
-		    "\nNSCreateObjectFileImageFromFile() error: %s",
-		    objFileImageErrMsg);
-	}
-#endif /* TCL_DYLD_USE_NSMODULE */
 	Tcl_SetObjResult(interp, errObj);
 	result = TCL_ERROR;
     }
@@ -355,14 +249,12 @@ FindSymbol(
     }
     native = Tcl_DStringValue(&ds);
     if (dyldLoadHandle->dlHandle) {
-#if TCL_DYLD_USE_DLFCN
 	proc = (Tcl_LibraryInitProc *)dlsym(dyldLoadHandle->dlHandle, native);
 	if (!proc) {
 	    errMsg = dlerror();
 	}
-#endif /* TCL_DYLD_USE_DLFCN */
     } else {
-#if TCL_DYLD_USE_NSMODULE || defined(TCL_LOAD_FROM_MEMORY)
+#if defined(TCL_LOAD_FROM_MEMORY)
 	NSSymbol nsSymbol = NULL;
 	Tcl_DString newName;
 
@@ -415,7 +307,7 @@ FindSymbol(
 	    proc = (Tcl_LibraryInitProc *)NSAddressOfSymbol(nsSymbol);
 	}
 	Tcl_DStringFree(&newName);
-#endif /* TCL_DYLD_USE_NSMODULE */
+#endif /* TCL_LOAD_FROM_MEMORY */
     }
     Tcl_DStringFree(&ds);
     if (errMsg && (interp != NULL)) {
@@ -456,11 +348,9 @@ UnloadFile(
     Tcl_DyldLoadHandle *dyldLoadHandle = (Tcl_DyldLoadHandle *)loadHandle->clientData;
 
     if (dyldLoadHandle->dlHandle) {
-#if TCL_DYLD_USE_DLFCN
 	(void) dlclose(dyldLoadHandle->dlHandle);
-#endif /* TCL_DYLD_USE_DLFCN */
     } else {
-#if TCL_DYLD_USE_NSMODULE || defined(TCL_LOAD_FROM_MEMORY)
+#if defined(TCL_LOAD_FROM_MEMORY)
 	Tcl_DyldModuleHandle *modulePtr = dyldLoadHandle->modulePtr;
 
 	while (modulePtr != NULL) {
@@ -471,7 +361,7 @@ UnloadFile(
 	    modulePtr = modulePtr->nextPtr;
 	    Tcl_Free(ptr);
 	}
-#endif /* TCL_DYLD_USE_NSMODULE */
+#endif /* TCL_LOAD_FROM_MEMORY */
     }
     Tcl_Free(dyldLoadHandle);
     Tcl_Free(loadHandle);
@@ -496,25 +386,18 @@ UnloadFile(
 #ifdef TCL_LOAD_FROM_MEMORY
 MODULE_SCOPE void *
 TclpLoadMemoryGetBuffer(
-    TCL_UNUSED(Tcl_Interp *),
-    int size)			/* Size of desired buffer. */
+    size_t size)			/* Size of desired buffer. */
 {
     void *buffer = NULL;
 
     /*
-     * NSCreateObjectFileImageFromMemory is available but always fails
-     * prior to Darwin 7.
+     * We must allocate the buffer using vm_allocate, because
+     * NSCreateObjectFileImageFromMemory will dispose of it using
+     * vm_deallocate.
      */
-    if (tclMacOSXDarwinRelease >= 7) {
-	/*
-	 * We must allocate the buffer using vm_allocate, because
-	 * NSCreateObjectFileImageFromMemory will dispose of it using
-	 * vm_deallocate.
-	 */
 
-	if (vm_allocate(mach_task_self(), (vm_address_t *) &buffer, size, 1)) {
-	    buffer = NULL;
-	}
+    if (vm_allocate(mach_task_self(), (vm_address_t *) &buffer, size, 1)) {
+	buffer = NULL;
     }
     return buffer;
 }
@@ -541,13 +424,13 @@ TclpLoadMemoryGetBuffer(
 #ifdef TCL_LOAD_FROM_MEMORY
 MODULE_SCOPE int
 TclpLoadMemory(
-    Tcl_Interp *interp,		/* Used for error reporting. */
     void *buffer,		/* Buffer containing the desired code
 				 * (allocated with TclpLoadMemoryGetBuffer). */
-    int size,			/* Allocation size of buffer. */
-    int codeSize,		/* Size of code data read into buffer or -1 if
+    size_t size,		/* Allocation size of buffer. */
+    Tcl_Size codeSize,	/* Size of code data read into buffer or -1 if
 				 * an error occurred and the buffer should
 				 * just be freed. */
+    const char *path,
     Tcl_LoadHandle *loadHandle, /* Filled with token for dynamically loaded
 				 * file which will be passed back to
 				 * (*unloadProcPtr)() to unload the file. */
@@ -585,7 +468,7 @@ TclpLoadMemory(
 #	define arch_abi CPU_ARCH_ABI64
 #endif /*  __LP64__ */
 
-	if ((size_t) codeSize >= sizeof(struct fat_header)
+	if ((size_t)codeSize >= sizeof(struct fat_header)
 		&& fh->magic == OSSwapHostToBigInt32(FAT_MAGIC)) {
 	    uint32_t fh_nfat_arch = OSSwapBigToHostInt32(fh->nfat_arch);
 
@@ -593,7 +476,7 @@ TclpLoadMemory(
 	     * Fat binary, try to find mach_header for our architecture
 	     */
 
-	    if ((size_t) codeSize >= sizeof(struct fat_header) +
+	    if ((size_t)codeSize >= sizeof(struct fat_header) +
 		    fh_nfat_arch * sizeof(struct fat_arch)) {
 		void *fatarchs = (char *)buffer + sizeof(struct fat_header);
 		const NXArchInfo *arch = NXGetLocalArchInfo();
@@ -641,16 +524,11 @@ TclpLoadMemory(
 
     /*
      * If it went wrong (or we were asked to just deallocate), get rid of the
-     * memory block and create an error message.
+     * memory block.
      */
 
     if (dyldObjFileImage == NULL) {
 	vm_deallocate(mach_task_self(), (vm_address_t) buffer, size);
-	if (objFileImageErrMsg != NULL) {
-	    Tcl_SetObjResult(interp, Tcl_ObjPrintf(
-		    "NSCreateObjectFileImageFromMemory() error: %s",
-		    objFileImageErrMsg));
-	}
 	return TCL_ERROR;
     }
 
@@ -664,7 +542,7 @@ TclpLoadMemory(
     if (!(flags & 2)) {
 	nsflags |= NSLINKMODULE_OPTION_BINDNOW;
     }
-    module = NSLinkModule(dyldObjFileImage, "[Memory Based Bundle]", nsflags);
+    module = NSLinkModule(dyldObjFileImage, (path ? path : "[Memory Based Bundle]"), nsflags);
     NSDestroyObjectFileImage(dyldObjFileImage);
     if (!module) {
 	NSLinkEditErrors editError;
@@ -672,7 +550,6 @@ TclpLoadMemory(
 	const char *errorName, *errMsg;
 
 	NSLinkEditError(&editError, &errorNumber, &errorName, &errMsg);
-	Tcl_SetObjResult(interp, Tcl_NewStringObj(errMsg, TCL_INDEX_NONE));
 	return TCL_ERROR;
     }
 
