@@ -3,13 +3,15 @@
  *
  *	Contains platform specific test commands for the Unix platform.
  *
- * Copyright (c) 1996-1997 Sun Microsystems, Inc.
- * Copyright (c) 1998 by Scriptics Corporation.
+ * Copyright © 1996-1997 Sun Microsystems, Inc.
+ * Copyright © 1998 Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
  */
 
+#undef BUILD_tcl
+#undef STATIC_BUILD
 #ifndef USE_TCL_STUBS
 #   define USE_TCL_STUBS
 #endif
@@ -37,7 +39,7 @@
  * exercised by the "testfilehandler" command.
  */
 
-typedef struct Pipe {
+typedef struct {
     TclFile readFile;		/* File handle for reading from the pipe. NULL
 				 * means pipe doesn't exist yet. */
     TclFile writeFile;		/* File handle for writing from the pipe. */
@@ -62,18 +64,15 @@ static const char *gotsig = "0";
  * Forward declarations of functions defined later in this file:
  */
 
-static Tcl_CmdProc TestalarmCmd;
+static Tcl_ObjCmdProc TestalarmCmd;
 static Tcl_ObjCmdProc TestchmodCmd;
-static Tcl_CmdProc TestfilehandlerCmd;
-static Tcl_CmdProc TestfilewaitCmd;
-static Tcl_CmdProc TestfindexecutableCmd;
-static Tcl_ObjCmdProc TestforkObjCmd;
-static Tcl_CmdProc TestgetdefencdirCmd;
-static Tcl_CmdProc TestgetopenfileCmd;
-static Tcl_CmdProc TestgotsigCmd;
-static Tcl_CmdProc TestsetdefencdirCmd;
+static Tcl_ObjCmdProc TestfilehandlerCmd;
+static Tcl_ObjCmdProc TestfilewaitCmd;
+static Tcl_ObjCmdProc TestfindexecutableCmd;
+static Tcl_ObjCmdProc TestforkCmd;
+static Tcl_ObjCmdProc TestgotsigCmd;
 static Tcl_FileProc TestFileHandlerProc;
-static void AlarmHandler(int signum);
+static void		AlarmHandler(int signum);
 
 /*
  *----------------------------------------------------------------------
@@ -98,23 +97,17 @@ TclplatformtestInit(
 {
     Tcl_CreateObjCommand(interp, "testchmod", TestchmodCmd,
 	    NULL, NULL);
-    Tcl_CreateCommand(interp, "testfilehandler", TestfilehandlerCmd,
+    Tcl_CreateObjCommand(interp, "testfilehandler", TestfilehandlerCmd,
 	    NULL, NULL);
-    Tcl_CreateCommand(interp, "testfilewait", TestfilewaitCmd,
+    Tcl_CreateObjCommand(interp, "testfilewait", TestfilewaitCmd,
 	    NULL, NULL);
-    Tcl_CreateCommand(interp, "testfindexecutable", TestfindexecutableCmd,
+    Tcl_CreateObjCommand(interp, "testfindexecutable", TestfindexecutableCmd,
 	    NULL, NULL);
-    Tcl_CreateObjCommand(interp, "testfork", TestforkObjCmd,
+    Tcl_CreateObjCommand(interp, "testfork", TestforkCmd,
 	NULL, NULL);
-    Tcl_CreateCommand(interp, "testgetopenfile", TestgetopenfileCmd,
+    Tcl_CreateObjCommand(interp, "testalarm", TestalarmCmd,
 	    NULL, NULL);
-    Tcl_CreateCommand(interp, "testgetdefenc", TestgetdefencdirCmd,
-	    NULL, NULL);
-    Tcl_CreateCommand(interp, "testsetdefenc", TestsetdefencdirCmd,
-	    NULL, NULL);
-    Tcl_CreateCommand(interp, "testalarm", TestalarmCmd,
-	    NULL, NULL);
-    Tcl_CreateCommand(interp, "testgotsig", TestgotsigCmd,
+    Tcl_CreateObjCommand(interp, "testgotsig", TestgotsigCmd,
 	    NULL, NULL);
     return TCL_OK;
 }
@@ -138,10 +131,10 @@ TclplatformtestInit(
 
 static int
 TestfilehandlerCmd(
-    ClientData clientData,	/* Not used. */
+    TCL_UNUSED(void *),
     Tcl_Interp *interp,		/* Current interpreter. */
-    int argc,			/* Number of arguments. */
-    const char **argv)		/* Argument strings. */
+    int objc,			/* Number of arguments. */
+    Tcl_Obj *const *objv)	/* Argument strings. */
 {
     Pipe *pipePtr;
     int i, mask, timeout;
@@ -161,24 +154,23 @@ TestfilehandlerCmd(
 	initialized = 1;
     }
 
-    if (argc < 2) {
-	Tcl_AppendResult(interp, "wrong # arguments: should be \"", argv[0],
-		" option ... \"", NULL);
+    if (objc < 2) {
+	Tcl_WrongNumArgs(interp, 1, objv, "option ...");
 	return TCL_ERROR;
     }
     pipePtr = NULL;
-    if (argc >= 3) {
-	if (Tcl_GetInt(interp, argv[2], &i) != TCL_OK) {
+    if (objc >= 3) {
+	if (Tcl_GetIntFromObj(interp, objv[2], &i) != TCL_OK) {
 	    return TCL_ERROR;
 	}
 	if (i >= MAX_PIPES) {
-	    Tcl_AppendResult(interp, "bad index ", argv[2], NULL);
+	    Tcl_AppendResult(interp, "bad index ", objv[2], (char *)NULL);
 	    return TCL_ERROR;
 	}
 	pipePtr = &testPipes[i];
     }
 
-    if (strcmp(argv[1], "close") == 0) {
+    if (strcmp(Tcl_GetString(objv[1]), "close") == 0) {
 	for (i = 0; i < MAX_PIPES; i++) {
 	    if (testPipes[i].readFile != NULL) {
 		TclpCloseFile(testPipes[i].readFile);
@@ -187,85 +179,80 @@ TestfilehandlerCmd(
 		testPipes[i].writeFile = NULL;
 	    }
 	}
-    } else if (strcmp(argv[1], "clear") == 0) {
-	if (argc != 3) {
-	    Tcl_AppendResult(interp, "wrong # arguments: should be \"",
-		    argv[0], " clear index\"", NULL);
+    } else if (strcmp(Tcl_GetString(objv[1]), "clear") == 0) {
+	if (objc != 3) {
+	    Tcl_WrongNumArgs(interp, 2, objv, "index");
 	    return TCL_ERROR;
 	}
 	pipePtr->readCount = pipePtr->writeCount = 0;
-    } else if (strcmp(argv[1], "counts") == 0) {
+    } else if (strcmp(Tcl_GetString(objv[1]), "counts") == 0) {
 	char buf[TCL_INTEGER_SPACE * 2];
 
-	if (argc != 3) {
-	    Tcl_AppendResult(interp, "wrong # arguments: should be \"",
-		    argv[0], " counts index\"", NULL);
+	if (objc != 3) {
+	    Tcl_WrongNumArgs(interp, 2, objv, "index");
 	    return TCL_ERROR;
 	}
 	snprintf(buf, sizeof(buf), "%d %d", pipePtr->readCount, pipePtr->writeCount);
-	Tcl_AppendResult(interp, buf, NULL);
-    } else if (strcmp(argv[1], "create") == 0) {
-	if (argc != 5) {
-	    Tcl_AppendResult(interp, "wrong # arguments: should be \"",
-		    argv[0], " create index readMode writeMode\"", NULL);
+	Tcl_AppendResult(interp, buf, (char *)NULL);
+    } else if (strcmp(Tcl_GetString(objv[1]), "create") == 0) {
+	if (objc != 5) {
+	    Tcl_WrongNumArgs(interp, 2, objv, "index readMode writeMode");
 	    return TCL_ERROR;
 	}
 	if (pipePtr->readFile == NULL) {
 	    if (!TclpCreatePipe(&pipePtr->readFile, &pipePtr->writeFile)) {
 		Tcl_AppendResult(interp, "couldn't open pipe: ",
-			Tcl_PosixError(interp), NULL);
+			Tcl_PosixError(interp), (char *)NULL);
 		return TCL_ERROR;
 	    }
 #ifdef O_NONBLOCK
 	    fcntl(GetFd(pipePtr->readFile), F_SETFL, O_NONBLOCK);
 	    fcntl(GetFd(pipePtr->writeFile), F_SETFL, O_NONBLOCK);
 #else
-	    Tcl_AppendResult(interp, "can't make pipes non-blocking",
-		    NULL);
+	    Tcl_AppendResult(interp, "cannot make pipes non-blocking",
+		    (char *)NULL);
 	    return TCL_ERROR;
 #endif
 	}
 	pipePtr->readCount = 0;
 	pipePtr->writeCount = 0;
 
-	if (strcmp(argv[3], "readable") == 0) {
+	if (strcmp(Tcl_GetString(objv[3]), "readable") == 0) {
 	    Tcl_CreateFileHandler(GetFd(pipePtr->readFile), TCL_READABLE,
 		    TestFileHandlerProc, pipePtr);
-	} else if (strcmp(argv[3], "off") == 0) {
+	} else if (strcmp(Tcl_GetString(objv[3]), "off") == 0) {
 	    Tcl_DeleteFileHandler(GetFd(pipePtr->readFile));
-	} else if (strcmp(argv[3], "disabled") == 0) {
+	} else if (strcmp(Tcl_GetString(objv[3]), "disabled") == 0) {
 	    Tcl_CreateFileHandler(GetFd(pipePtr->readFile), 0,
 		    TestFileHandlerProc, pipePtr);
 	} else {
-	    Tcl_AppendResult(interp, "bad read mode \"", argv[3], "\"", NULL);
+	    Tcl_AppendResult(interp, "bad read mode \"", Tcl_GetString(objv[3]), "\"", (char *)NULL);
 	    return TCL_ERROR;
 	}
-	if (strcmp(argv[4], "writable") == 0) {
+	if (strcmp(Tcl_GetString(objv[4]), "writable") == 0) {
 	    Tcl_CreateFileHandler(GetFd(pipePtr->writeFile), TCL_WRITABLE,
 		    TestFileHandlerProc, pipePtr);
-	} else if (strcmp(argv[4], "off") == 0) {
+	} else if (strcmp(Tcl_GetString(objv[4]), "off") == 0) {
 	    Tcl_DeleteFileHandler(GetFd(pipePtr->writeFile));
-	} else if (strcmp(argv[4], "disabled") == 0) {
+	} else if (strcmp(Tcl_GetString(objv[4]), "disabled") == 0) {
 	    Tcl_CreateFileHandler(GetFd(pipePtr->writeFile), 0,
 		    TestFileHandlerProc, pipePtr);
 	} else {
-	    Tcl_AppendResult(interp, "bad read mode \"", argv[4], "\"", NULL);
+	    Tcl_AppendResult(interp, "bad read mode \"", Tcl_GetString(objv[4]), "\"", (char *)NULL);
 	    return TCL_ERROR;
 	}
-    } else if (strcmp(argv[1], "empty") == 0) {
-	if (argc != 3) {
-	    Tcl_AppendResult(interp, "wrong # arguments: should be \"",
-		    argv[0], " empty index\"", NULL);
+    } else if (strcmp(Tcl_GetString(objv[1]), "empty") == 0) {
+	if (objc != 3) {
+	    Tcl_WrongNumArgs(interp, 2, objv, "index");
 	    return TCL_ERROR;
 	}
 
 	while (read(GetFd(pipePtr->readFile), buffer, 4000) > 0) {
 	    /* Empty loop body. */
 	}
-    } else if (strcmp(argv[1], "fill") == 0) {
-	if (argc != 3) {
-	    Tcl_AppendResult(interp, "wrong # arguments: should be \"",
-		    argv[0], " fill index\"", NULL);
+    } else if (strcmp(Tcl_GetString(objv[1]), "fill") == 0) {
+	if (objc != 3) {
+	    Tcl_WrongNumArgs(interp, 2, objv, "index");
 	    return TCL_ERROR;
 	}
 
@@ -273,38 +260,36 @@ TestfilehandlerCmd(
 	while (write(GetFd(pipePtr->writeFile), buffer, 4000) > 0) {
 	    /* Empty loop body. */
 	}
-    } else if (strcmp(argv[1], "fillpartial") == 0) {
+    } else if (strcmp(Tcl_GetString(objv[1]), "fillpartial") == 0) {
 	char buf[TCL_INTEGER_SPACE];
 
-	if (argc != 3) {
-	    Tcl_AppendResult(interp, "wrong # arguments: should be \"",
-		    argv[0], " fillpartial index\"", NULL);
+	if (objc != 3) {
+	    Tcl_WrongNumArgs(interp, 2, objv, "index");
 	    return TCL_ERROR;
 	}
 
 	memset(buffer, 'b', 10);
 	TclFormatInt(buf, write(GetFd(pipePtr->writeFile), buffer, 10));
-	Tcl_AppendResult(interp, buf, NULL);
-    } else if (strcmp(argv[1], "oneevent") == 0) {
+	Tcl_AppendResult(interp, buf, (char *)NULL);
+    } else if (strcmp(Tcl_GetString(objv[1]), "oneevent") == 0) {
 	Tcl_DoOneEvent(TCL_FILE_EVENTS|TCL_DONT_WAIT);
-    } else if (strcmp(argv[1], "wait") == 0) {
-	if (argc != 5) {
-	    Tcl_AppendResult(interp, "wrong # arguments: should be \"",
-		    argv[0], " wait index readable|writable timeout\"", NULL);
+    } else if (strcmp(Tcl_GetString(objv[1]), "wait") == 0) {
+	if (objc != 5) {
+	    Tcl_WrongNumArgs(interp, 2, objv, "index readable|writable timeout");
 	    return TCL_ERROR;
 	}
 	if (pipePtr->readFile == NULL) {
-	    Tcl_AppendResult(interp, "pipe ", argv[2], " doesn't exist", NULL);
+	    Tcl_AppendResult(interp, "pipe ", Tcl_GetString(objv[2]), " doesn't exist", (char *)NULL);
 	    return TCL_ERROR;
 	}
-	if (strcmp(argv[3], "readable") == 0) {
+	if (strcmp(Tcl_GetString(objv[3]), "readable") == 0) {
 	    mask = TCL_READABLE;
 	    file = pipePtr->readFile;
 	} else {
 	    mask = TCL_WRITABLE;
 	    file = pipePtr->writeFile;
 	}
-	if (Tcl_GetInt(interp, argv[4], &timeout) != TCL_OK) {
+	if (Tcl_GetIntFromObj(interp, objv[4], &timeout) != TCL_OK) {
 	    return TCL_ERROR;
 	}
 	i = TclUnixWaitForFile(GetFd(file), mask, timeout);
@@ -314,12 +299,12 @@ TestfilehandlerCmd(
 	if (i & TCL_WRITABLE) {
 	    Tcl_AppendElement(interp, "writable");
 	}
-    } else if (strcmp(argv[1], "windowevent") == 0) {
+    } else if (strcmp(Tcl_GetString(objv[1]), "windowevent") == 0) {
 	Tcl_DoOneEvent(TCL_WINDOW_EVENTS|TCL_DONT_WAIT);
     } else {
-	Tcl_AppendResult(interp, "bad option \"", argv[1],
+	Tcl_AppendResult(interp, "bad option \"", Tcl_GetString(objv[1]),
 		"\": must be close, clear, counts, create, empty, fill, "
-		"fillpartial, oneevent, wait, or windowevent", NULL);
+		"fillpartial, oneevent, wait, or windowevent", (char *)NULL);
 	return TCL_ERROR;
     }
     return TCL_OK;
@@ -327,11 +312,11 @@ TestfilehandlerCmd(
 
 static void
 TestFileHandlerProc(
-    ClientData clientData,	/* Points to a Pipe structure. */
+    void *clientData,	/* Points to a Pipe structure. */
     int mask)			/* Indicates which events happened:
 				 * TCL_READABLE or TCL_WRITABLE. */
 {
-    Pipe *pipePtr = clientData;
+    Pipe *pipePtr = (Pipe *)clientData;
 
     if (mask & TCL_READABLE) {
 	pipePtr->readCount++;
@@ -360,44 +345,43 @@ TestFileHandlerProc(
 
 static int
 TestfilewaitCmd(
-    ClientData clientData,	/* Not used. */
+    TCL_UNUSED(void *),
     Tcl_Interp *interp,		/* Current interpreter. */
-    int argc,			/* Number of arguments. */
-    const char **argv)		/* Argument strings. */
+    int objc,			/* Number of arguments. */
+    Tcl_Obj *const *objv)	/* Argument strings. */
 {
     int mask, result, timeout;
     Tcl_Channel channel;
     int fd;
-    ClientData data;
+    void *data;
 
-    if (argc != 4) {
-	Tcl_AppendResult(interp, "wrong # arguments: should be \"", argv[0],
-		" file readable|writable|both timeout\"", NULL);
+    if (objc != 4) {
+	Tcl_WrongNumArgs(interp, 2, objv, "file readable|writable|both timeout");
 	return TCL_ERROR;
     }
-    channel = Tcl_GetChannel(interp, argv[1], NULL);
+    channel = Tcl_GetChannel(interp, Tcl_GetString(objv[1]), NULL);
     if (channel == NULL) {
 	return TCL_ERROR;
     }
-    if (strcmp(argv[2], "readable") == 0) {
+    if (strcmp(Tcl_GetString(objv[2]), "readable") == 0) {
 	mask = TCL_READABLE;
-    } else if (strcmp(argv[2], "writable") == 0){
+    } else if (strcmp(Tcl_GetString(objv[2]), "writable") == 0){
 	mask = TCL_WRITABLE;
-    } else if (strcmp(argv[2], "both") == 0){
+    } else if (strcmp(Tcl_GetString(objv[2]), "both") == 0){
 	mask = TCL_WRITABLE|TCL_READABLE;
     } else {
-	Tcl_AppendResult(interp, "bad argument \"", argv[2],
-		"\": must be readable, writable, or both", NULL);
+	Tcl_AppendResult(interp, "bad argument \"", Tcl_GetString(objv[2]),
+		"\": must be readable, writable, or both", (char *)NULL);
 	return TCL_ERROR;
     }
     if (Tcl_GetChannelHandle(channel,
 	    (mask & TCL_READABLE) ? TCL_READABLE : TCL_WRITABLE,
-	    (ClientData*) &data) != TCL_OK) {
-	Tcl_AppendResult(interp, "couldn't get channel file", NULL);
+	    (void **) &data) != TCL_OK) {
+	Tcl_AppendResult(interp, "couldn't get channel file", (char *)NULL);
 	return TCL_ERROR;
     }
     fd = PTR2INT(data);
-    if (Tcl_GetInt(interp, argv[3], &timeout) != TCL_OK) {
+    if (Tcl_GetIntFromObj(interp, objv[3], &timeout) != TCL_OK) {
 	return TCL_ERROR;
     }
     result = TclUnixWaitForFile(fd, mask, timeout);
@@ -429,23 +413,22 @@ TestfilewaitCmd(
 
 static int
 TestfindexecutableCmd(
-    ClientData clientData,	/* Not used. */
+    TCL_UNUSED(void *),
     Tcl_Interp *interp,		/* Current interpreter. */
-    int argc,			/* Number of arguments. */
-    const char **argv)		/* Argument strings. */
+    int objc,			/* Number of arguments. */
+    Tcl_Obj *const *objv)	/* Argument strings. */
 {
     Tcl_Obj *saveName;
 
-    if (argc != 2) {
-	Tcl_AppendResult(interp, "wrong # arguments: should be \"", argv[0],
-		" argv0\"", NULL);
+    if (objc != 2) {
+	Tcl_WrongNumArgs(interp, 1, objv, "argv0");
 	return TCL_ERROR;
     }
 
     saveName = TclGetObjNameOfExecutable();
     Tcl_IncrRefCount(saveName);
 
-    TclpFindExecutable(argv[1]);
+    TclpFindExecutable(Tcl_GetString(objv[1]));
     Tcl_SetObjResult(interp, TclGetObjNameOfExecutable());
 
     TclSetObjNameOfExecutable(saveName, NULL);
@@ -456,84 +439,7 @@ TestfindexecutableCmd(
 /*
  *----------------------------------------------------------------------
  *
- * TestgetopenfileCmd --
- *
- *	This function implements the "testgetopenfile" command. It is used to
- *	get a FILE * value from a registered channel.
- *
- * Results:
- *	A standard Tcl result.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-static int
-TestgetopenfileCmd(
-    ClientData clientData,	/* Not used. */
-    Tcl_Interp *interp,		/* Current interpreter. */
-    int argc,			/* Number of arguments. */
-    const char **argv)		/* Argument strings. */
-{
-    ClientData filePtr;
-
-    if (argc != 3) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		" channelName forWriting\"", NULL);
-	return TCL_ERROR;
-    }
-    if (Tcl_GetOpenFile(interp, argv[1], atoi(argv[2]), 1, &filePtr)
-	    == TCL_ERROR) {
-	return TCL_ERROR;
-    }
-    if (filePtr == NULL) {
-	Tcl_AppendResult(interp,
-		"Tcl_GetOpenFile succeeded but FILE * NULL!", NULL);
-	return TCL_ERROR;
-    }
-    return TCL_OK;
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * TestsetdefencdirCmd --
- *
- *	This function implements the "testsetdefenc" command. It is used to
- *	test Tcl_SetDefaultEncodingDir().
- *
- * Results:
- *	A standard Tcl result.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-static int
-TestsetdefencdirCmd(
-    ClientData clientData,	/* Not used. */
-    Tcl_Interp *interp,		/* Current interpreter. */
-    int argc,			/* Number of arguments. */
-    const char **argv)		/* Argument strings. */
-{
-    if (argc != 2) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0],
-		" defaultDir\"", NULL);
-	return TCL_ERROR;
-    }
-
-    Tcl_SetDefaultEncodingDir(argv[1]);
-    return TCL_OK;
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * TestforkObjCmd --
+ * TestforkCmd --
  *
  *	This function implements the "testfork" command. It is used to
  *	fork the Tcl process for specific test cases.
@@ -548,11 +454,11 @@ TestsetdefencdirCmd(
  */
 
 static int
-TestforkObjCmd(
-    ClientData clientData,	/* Not used. */
+TestforkCmd(
+    TCL_UNUSED(void *),
     Tcl_Interp *interp,		/* Current interpreter. */
     int objc,			/* Number of arguments. */
-    Tcl_Obj *const *objv)		/* Argument strings. */
+    Tcl_Obj *const *objv)	/* Argument strings. */
 {
     pid_t pid;
 
@@ -563,7 +469,7 @@ TestforkObjCmd(
     pid = fork();
     if (pid == -1) {
 	Tcl_AppendResult(interp,
-		"Cannot fork", NULL);
+		"Cannot fork", (char *)NULL);
 	return TCL_ERROR;
     }
     /* Only needed when pthread_atfork is not present,
@@ -571,40 +477,7 @@ TestforkObjCmd(
     if (pid==0) {
 	Tcl_InitNotifier();
     }
-    Tcl_SetObjResult(interp, Tcl_NewIntObj(pid));
-    return TCL_OK;
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * TestgetdefencdirCmd --
- *
- *	This function implements the "testgetdefenc" command. It is used to
- *	test Tcl_GetDefaultEncodingDir().
- *
- * Results:
- *	A standard Tcl result.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-static int
-TestgetdefencdirCmd(
-    ClientData clientData,	/* Not used. */
-    Tcl_Interp *interp,		/* Current interpreter. */
-    int argc,			/* Number of arguments. */
-    const char **argv)		/* Argument strings. */
-{
-    if (argc != 1) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"", argv[0], NULL);
-	return TCL_ERROR;
-    }
-
-    Tcl_AppendResult(interp, Tcl_GetDefaultEncodingDir(), NULL);
+    Tcl_SetObjResult(interp, Tcl_NewWideIntObj(pid));
     return TCL_OK;
 }
 
@@ -628,19 +501,17 @@ TestgetdefencdirCmd(
 
 static int
 TestalarmCmd(
-    ClientData clientData,	/* Not used. */
+    TCL_UNUSED(void *),
     Tcl_Interp *interp,		/* Current interpreter. */
-    int argc,			/* Number of arguments. */
-    const char **argv)		/* Argument strings. */
+    int objc,			/* Number of arguments. */
+    Tcl_Obj *const *objv)	/* Argument strings. */
 {
 #ifdef SA_RESTART
-    unsigned int sec;
+    unsigned int sec = 1;
     struct sigaction action;
 
-    if (argc > 1) {
-	Tcl_GetInt(interp, argv[1], (int *)&sec);
-    } else {
-	sec = 1;
+    if (objc > 1) {
+	Tcl_GetIntFromObj(interp, objv[1], (int *)&sec);
     }
 
     /*
@@ -649,19 +520,20 @@ TestalarmCmd(
      */
 
     action.sa_handler = AlarmHandler;
-    memset((void *) &action.sa_mask, 0, sizeof(sigset_t));
+    memset((void *)&action.sa_mask, 0, sizeof(sigset_t));
     action.sa_flags = SA_RESTART;
 
     if (sigaction(SIGALRM, &action, NULL) < 0) {
-	Tcl_AppendResult(interp, "sigaction: ", Tcl_PosixError(interp), NULL);
+	Tcl_AppendResult(interp, "sigaction: ", Tcl_PosixError(interp), (char *)NULL);
 	return TCL_ERROR;
     }
     (void) alarm(sec);
     return TCL_OK;
 #else
+
     Tcl_AppendResult(interp,
 	    "warning: sigaction SA_RESTART not support on this platform",
-	    NULL);
+	    (char *)NULL);
     return TCL_ERROR;
 #endif
 }
@@ -677,14 +549,14 @@ TestalarmCmd(
  *	None.
  *
  * Side effects:
- * 	Calls the Tcl Async handler.
+ *	Calls the Tcl Async handler.
  *
  *----------------------------------------------------------------------
  */
 
 static void
 AlarmHandler(
-    int signum)
+    TCL_UNUSED(int) /*signum*/)
 {
     gotsig = "1";
 }
@@ -694,7 +566,7 @@ AlarmHandler(
  *
  * TestgotsigCmd --
  *
- * 	Verify the signal was handled after the testalarm command.
+ *	Verify the signal was handled after the testalarm command.
  *
  * Results:
  *	None.
@@ -707,12 +579,12 @@ AlarmHandler(
 
 static int
 TestgotsigCmd(
-    ClientData clientData,	/* Not used. */
+    TCL_UNUSED(void *),
     Tcl_Interp *interp,		/* Current interpreter. */
-    int argc,			/* Number of arguments. */
-    const char **argv)		/* Argument strings. */
+    TCL_UNUSED(int) /*objc*/,
+    TCL_UNUSED(Tcl_Obj *const *))
 {
-    Tcl_AppendResult(interp, gotsig, NULL);
+    Tcl_AppendResult(interp, gotsig, (char *)NULL);
     gotsig = "0";
     return TCL_OK;
 }
@@ -738,7 +610,7 @@ TestgotsigCmd(
 
 static int
 TestchmodCmd(
-    ClientData dummy,			/* Not used. */
+    TCL_UNUSED(void *),
     Tcl_Interp *interp,			/* Current interpreter. */
     int objc,			/* Number of arguments. */
     Tcl_Obj *const *objv)		/* Argument strings. */
@@ -762,9 +634,9 @@ TestchmodCmd(
 	if (translated == NULL) {
 	    return TCL_ERROR;
 	}
-	if (chmod(translated, (unsigned) mode) != 0) {
+	if (chmod(translated, mode) != 0) {
 	    Tcl_AppendResult(interp, translated, ": ", Tcl_PosixError(interp),
-		    NULL);
+		    (char *)NULL);
 	    return TCL_ERROR;
 	}
 	Tcl_DStringFree(&buffer);
