@@ -32,25 +32,27 @@ TCL_DECLARE_MUTEX(serialMutex)
 /*
  * Bit masks used in the flags field of the SerialInfo structure below.
  */
+enum SerialFlags {
+    SERIAL_PENDING = 1 << 0,	/* Message is pending in the queue. */
+    SERIAL_ASYNC = 1 << 1,	/* Channel is non-blocking. */
 
-#define SERIAL_PENDING	(1<<0)	/* Message is pending in the queue. */
-#define SERIAL_ASYNC	(1<<1)	/* Channel is non-blocking. */
+    /*
+     * Bit masks used in the sharedFlags field of the SerialInfo structure
+     * below.
+     */
 
-/*
- * Bit masks used in the sharedFlags field of the SerialInfo structure below.
- */
+    SERIAL_EOF = 1 << 2,	/* Serial has reached EOF. */
+    SERIAL_ERROR = 1 << 4,
 
-#define SERIAL_EOF	(1<<2)	/* Serial has reached EOF. */
-#define SERIAL_ERROR	(1<<4)
+    /*
+     * Bit masks used for noting whether to drain or discard output on close.
+     * They are disjoint from each other; at most one may be set at a time.
+     */
 
-/*
- * Bit masks used for noting whether to drain or discard output on close. They
- * are disjoint from each other; at most one may be set at a time.
- */
-
-#define SERIAL_CLOSE_DRAIN   (1<<6)	/* Drain all output on close. */
-#define SERIAL_CLOSE_DISCARD (1<<7)	/* Discard all output on close. */
-#define SERIAL_CLOSE_MASK    (3<<6)	/* Both two bits above. */
+    SERIAL_CLOSE_DRAIN = 1<<6,	/* Drain all output on close. */
+    SERIAL_CLOSE_DISCARD = 1<<7,/* Discard all output on close. */
+    SERIAL_CLOSE_MASK = 3<<6	/* Both two bits above. */
+};
 
 /*
  * Default time to block between checking status on the serial port.
@@ -59,13 +61,14 @@ TCL_DECLARE_MUTEX(serialMutex)
 #define SERIAL_DEFAULT_BLOCKTIME 10	/* 10 msec */
 
 /*
- * Define Win32 read/write error masks returned by ClearCommError()
+ * Win32 read/write error masks for values returned by ClearCommError()
  */
-
-#define SERIAL_READ_ERRORS \
-	(CE_RXOVER | CE_OVERRUN | CE_RXPARITY | CE_FRAME  | CE_BREAK)
-#define SERIAL_WRITE_ERRORS \
+enum TclWinCommErrorMasks {
+    SERIAL_READ_ERRORS =	/* Errors in the reader side. */
+	(CE_RXOVER | CE_OVERRUN | CE_RXPARITY | CE_FRAME  | CE_BREAK),
+    SERIAL_WRITE_ERRORS =	/* Errors in the writer side. */
 	(CE_TXFULL | CE_PTO)
+};
 
 /*
  * This structure describes per-instance data for a serial based channel.
@@ -85,7 +88,8 @@ typedef struct SerialInfo {
     int readable;		/* Flag that the channel is readable. */
     int writable;		/* Flag that the channel is writable. */
     int blockTime;		/* Maximum blocktime in msec. */
-    unsigned long long lastEventTime;	/* Time in milliseconds since last readable
+    unsigned long long lastEventTime;
+				/* Time in milliseconds since last readable
 				 * event. */
 				/* Next readable event only after blockTime */
     DWORD error;		/* pending error code returned by
@@ -380,7 +384,8 @@ SerialGetMilliseconds(void)
 
     Tcl_GetTime(&time);
 
-    return ((unsigned long long)time.sec * 1000 + (unsigned long)time.usec / 1000);
+    return (unsigned long long)time.sec * 1000
+	    + (unsigned long)time.usec / 1000;
 }
 
 /*
@@ -561,7 +566,7 @@ SerialCheckProc(
 
 static int
 SerialBlockProc(
-    void *instanceData,    /* Instance data for channel. */
+    void *instanceData,		/* Instance data for channel. */
     int mode)			/* TCL_MODE_BLOCKING or
 				 * TCL_MODE_NONBLOCKING. */
 {
@@ -600,7 +605,7 @@ SerialBlockProc(
 
 static int
 SerialCloseProc(
-    void *instanceData,    /* Pointer to SerialInfo structure. */
+    void *instanceData,		/* Pointer to SerialInfo structure. */
     TCL_UNUSED(Tcl_Interp *),
     int flags)
 {
@@ -854,7 +859,7 @@ SerialBlockingWrite(
 
 static int
 SerialInputProc(
-    void *instanceData,	/* Serial state. */
+    void *instanceData,		/* Serial state. */
     char *buf,			/* Where to store data read. */
     int bufSize,		/* How much space is available in the
 				 * buffer? */
@@ -961,7 +966,7 @@ SerialInputProc(
 
 static int
 SerialOutputProc(
-    void *instanceData,	/* Serial state. */
+    void *instanceData,		/* Serial state. */
     const char *buf,		/* The data buffer. */
     int toWrite,		/* How many bytes to write? */
     int *errorCode)		/* Where to store error code. */
@@ -1191,7 +1196,7 @@ SerialEventProc(
 
 static void
 SerialWatchProc(
-    void *instanceData,	/* Serial state. */
+    void *instanceData,		/* Serial state. */
     int mask)			/* What events to watch for, OR-ed combination
 				 * of TCL_READABLE, TCL_WRITABLE and
 				 * TCL_EXCEPTION. */
@@ -1248,9 +1253,9 @@ SerialWatchProc(
 
 static int
 SerialGetHandleProc(
-    void *instanceData,	/* The serial state. */
+    void *instanceData,		/* The serial state. */
     TCL_UNUSED(int) /*direction*/,
-    void **handlePtr)	/* Where to store the handle. */
+    void **handlePtr)		/* Where to store the handle. */
 {
     SerialInfo *infoPtr = (SerialInfo *) instanceData;
 
@@ -1280,7 +1285,7 @@ SerialWriterThread(
     LPVOID arg)
 {
     TclPipeThreadInfo *pipeTI = (TclPipeThreadInfo *)arg;
-    SerialInfo *infoPtr = NULL; /* access info only after success init/wait */
+    SerialInfo *infoPtr = NULL;	/* access info only after success init/wait */
     DWORD bytesWritten, toWrite;
     char *buf;
     OVERLAPPED myWrite;		/* Have an own OVERLAPPED in this thread. */
@@ -1610,12 +1615,12 @@ SerialModemStatusStr(
 
 static int
 SerialSetOptionProc(
-    void *instanceData,	/* File state. */
+    void *instanceData,		/* Serial state. */
     Tcl_Interp *interp,		/* For error reporting - can be NULL. */
     const char *optionName,	/* Which option to set? */
     const char *value)		/* New value for option. */
 {
-    SerialInfo *infoPtr;
+    SerialInfo *infoPtr = (SerialInfo *) instanceData;
     DCB dcb;
     BOOL result, flag;
     size_t len, vlen;
@@ -1624,8 +1629,6 @@ SerialSetOptionProc(
     Tcl_Size argc;
     const char **argv;
 
-    infoPtr = (SerialInfo *) instanceData;
-
     /*
      * Parse options. This would be far easier if we had Tcl_Objs to work with
      * as that would let us use Tcl_GetIndexFromObj()...
@@ -1770,7 +1773,8 @@ SerialSetOptionProc(
 	    if (interp != NULL) {
 		Tcl_SetObjResult(interp, Tcl_NewStringObj(
 			"bad value for -xchar: should be a list of"
-			" two elements with each a single 8-bit character", TCL_INDEX_NONE));
+			" two elements with each a single 8-bit character",
+			TCL_INDEX_NONE));
 		Tcl_SetErrorCode(interp, "TCL", "VALUE", "XCHAR", (char *)NULL);
 	    }
 	    Tcl_Free((void *)argv);
@@ -2034,18 +2038,16 @@ SerialSetOptionProc(
 
 static int
 SerialGetOptionProc(
-    void *instanceData,	/* File state. */
+    void *instanceData,		/* Serial state. */
     Tcl_Interp *interp,		/* For error reporting - can be NULL. */
     const char *optionName,	/* Option to get. */
     Tcl_DString *dsPtr)		/* Where to store value(s). */
 {
-    SerialInfo *infoPtr;
+    SerialInfo *infoPtr = (SerialInfo *) instanceData;
     DCB dcb;
     size_t len;
     int valid = 0;		/* Flag if valid option parsed. */
 
-    infoPtr = (SerialInfo *) instanceData;
-
     if (optionName == NULL) {
 	len = 0;
     } else {
