@@ -172,16 +172,16 @@ struct TcpState {
  */
 
 enum TcpStateFlags {
-    TCP_NONBLOCKING = (1<<0),	/* Socket with non-blocking I/O. */
-    TCP_ASYNC_CONNECT = (1<<1),	/* Async connect in progress. */
-    SOCKET_EOF = (1<<2),	/* A zero read happened on the socket. */
-    SOCKET_PENDING = (1<<3),	/* A message has been sent for this socket */
-    TCP_ASYNC_PENDING = (1<<4),	/* TcpConnect was called to process an async
+    TCP_NONBLOCKING = 1<<0,	/* Socket with non-blocking I/O. */
+    TCP_ASYNC_CONNECT = 1<<1,	/* Async connect in progress. */
+    SOCKET_EOF = 1<<2,		/* A zero read happened on the socket. */
+    SOCKET_PENDING = 1<<3,	/* A message has been sent for this socket */
+    TCP_ASYNC_PENDING = 1<<4,	/* TcpConnect was called to process an async
 				 * connect. This flag indicates that reentry is
 				 * still pending. */
-    TCP_ASYNC_FAILED = (1<<5),	/* An async connect finally failed. */
+    TCP_ASYNC_FAILED = 1<<5,	/* An async connect finally failed. */
 
-    TCP_ASYNC_TEST_MODE = (1<<8)/* Async testing activated.  Do not
+    TCP_ASYNC_TEST_MODE = 1<<8	/* Async testing activated.  Do not
 				 * automatically continue connection
 				 * process */
 };
