@@ -4,7 +4,7 @@
  *	Contains Windows specific versions of Tcl functions that obtain time
  *	values from the operating system.
  *
- * Copyright 1995-1998 by Sun Microsystems, Inc.
+ * Copyright © 1995-1998 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution of
  * this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -12,52 +12,30 @@
 
 #include "tclInt.h"
 
-#define SECSPERDAY	(60L * 60L * 24L)
-#define SECSPERYEAR	(SECSPERDAY * 365L)
-#define SECSPER4YEAR	(SECSPERYEAR * 4L + SECSPERDAY)
-
 /*
  * Number of samples over which to estimate the performance counter.
  */
 
 #define SAMPLES		64
 
-/*
- * The following arrays contain the day of year for the last day of each
- * month, where index 1 is January.
- */
-
-static const int normalDays[] = {
-    -1, 30, 58, 89, 119, 150, 180, 211, 242, 272, 303, 333, 364
-};
-
-static const int leapDays[] = {
-    -1, 30, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365
-};
-
-typedef struct ThreadSpecificData {
-    char tzName[64];		/* Time zone name */
-    struct tm tm;		/* time information */
-} ThreadSpecificData;
-static Tcl_ThreadDataKey dataKey;
-
 /*
  * Data for managing high-resolution timers.
  */
 
-typedef struct TimeInfo {
+typedef struct {
     CRITICAL_SECTION cs;	/* Mutex guarding this structure. */
     int initialized;		/* Flag == 1 if this structure is
 				 * initialized. */
     int perfCounterAvailable;	/* Flag == 1 if the hardware has a performance
 				 * counter. */
-    DWORD calibrationInterv;	/* Calibration interval in seconds (start 1 sec) */
+    DWORD calibrationInterv;	/* Calibration interval in seconds (start 1
+				 * sec) */
     HANDLE calibrationThread;	/* Handle to the thread that keeps the virtual
 				 * clock calibrated. */
     HANDLE readyEvent;		/* System event used to trigger the requesting
 				 * thread when the clock calibration procedure
 				 * is initialized for the first time. */
-    HANDLE exitEvent; 		/* Event to signal out of an exit handler to
+    HANDLE exitEvent;		/* Event to signal out of an exit handler to
 				 * tell the calibration loop to terminate. */
     LARGE_INTEGER nominalFreq;	/* Nominal frequency of the system performance
 				 * counter, that is, the value returned from
@@ -81,9 +59,9 @@ typedef struct TimeInfo {
      * Data used in developing the estimate of performance counter frequency
      */
 
-    Tcl_WideUInt fileTimeSample[SAMPLES];
+    unsigned long long fileTimeSample[SAMPLES];
 				/* Last 64 samples of system time. */
-    Tcl_WideInt perfCounterSample[SAMPLES];
+    long long perfCounterSample[SAMPLES];
 				/* Last 64 samples of performance counter. */
     int sampleNo;		/* Current sample number. */
 } TimeInfo;
@@ -96,18 +74,18 @@ static TimeInfo timeInfo = {
     (HANDLE) NULL,
     (HANDLE) NULL,
     (HANDLE) NULL,
-#ifdef HAVE_CAST_TO_UNION
-    (LARGE_INTEGER) (Tcl_WideInt) 0,
+#if defined(HAVE_CAST_TO_UNION) && !defined(__cplusplus)
+    (LARGE_INTEGER) (long long) 0,
     (ULARGE_INTEGER) (DWORDLONG) 0,
-    (LARGE_INTEGER) (Tcl_WideInt) 0,
-    (LARGE_INTEGER) (Tcl_WideInt) 0,
-    (LARGE_INTEGER) (Tcl_WideInt) 0,
+    (LARGE_INTEGER) (long long) 0,
+    (LARGE_INTEGER) (long long) 0,
+    (LARGE_INTEGER) (long long) 0,
 #else
-    {0, 0},
-    {0, 0},
-    {0, 0},
-    {0, 0},
-    {0, 0},
+    {{0, 0}},
+    {{0, 0}},
+    {{0, 0}},
+    {{0, 0}},
+    {{0, 0}},
 #endif
     { 0 },
     { 0 },
@@ -125,24 +103,22 @@ static struct {
     double microsecsScale;	/* Denominator scale between clock / microsecs */
 } wideClick = {0, 0, 0.0};
 
-
 /*
  * Declarations for functions defined later in this file.
  */
 
-static struct tm *	ComputeGMT(const time_t *tp);
-static void		StopCalibration(ClientData clientData);
+static void		StopCalibration(void *clientData);
 static DWORD WINAPI	CalibrationThread(LPVOID arg);
-static void 		UpdateTimeEachSecond(void);
-static void		ResetCounterSamples(Tcl_WideUInt fileTime,
-			    Tcl_WideInt perfCounter, Tcl_WideInt perfFreq);
-static Tcl_WideInt	AccumulateSample(Tcl_WideInt perfCounter,
-			    Tcl_WideUInt fileTime);
+static void		UpdateTimeEachSecond(void);
+static void		ResetCounterSamples(unsigned long long fileTime,
+			    long long perfCounter, long long perfFreq);
+static long long	AccumulateSample(long long perfCounter,
+			    unsigned long long fileTime);
 static void		NativeScaleTime(Tcl_Time* timebuf,
-			    ClientData clientData);
-static Tcl_WideInt	NativeGetMicroseconds(void);
+			    void *clientData);
+static long long	NativeGetMicroseconds(void);
 static void		NativeGetTime(Tcl_Time* timebuf,
-			    ClientData clientData);
+			    void *clientData);
 
 /*
  * TIP #233 (Virtualized Time): Data for the time hooks, if any.
@@ -150,7 +126,24 @@ static void		NativeGetTime(Tcl_Time* timebuf,
 
 Tcl_GetTimeProc *tclGetTimeProcPtr = NativeGetTime;
 Tcl_ScaleTimeProc *tclScaleTimeProcPtr = NativeScaleTime;
-ClientData tclTimeClientData = NULL;
+void *tclTimeClientData = NULL;
+
+/*
+ * Inlined version of Tcl_GetTime.
+ */
+
+static inline void
+GetTime(
+    Tcl_Time *timePtr)
+{
+    tclGetTimeProcPtr(timePtr, tclTimeClientData);
+}
+
+static inline int
+IsTimeNative(void)
+{
+    return tclGetTimeProcPtr == NativeGetTime;
+}
 
 /*
  *----------------------------------------------------------------------
@@ -169,21 +162,22 @@ ClientData tclTimeClientData = NULL;
  *----------------------------------------------------------------------
  */
 
-unsigned long
+unsigned long long
 TclpGetSeconds(void)
 {
-    Tcl_WideInt usecSincePosixEpoch;
+    long long usecSincePosixEpoch;
 
-    /* Try to use high resolution timer */
-    if ( tclGetTimeProcPtr == NativeGetTime
-      && (usecSincePosixEpoch = NativeGetMicroseconds())
-    ) {
+    /*
+     * Try to use high resolution timer
+     */
+
+    if (IsTimeNative() && (usecSincePosixEpoch = NativeGetMicroseconds())) {
 	return usecSincePosixEpoch / 1000000;
     } else {
 	Tcl_Time t;
 
-	tclGetTimeProcPtr(&t, tclTimeClientData);	/* Tcl_GetTime inlined. */
-	return t.sec;
+	GetTime(&t);
+	return (unsigned long long)t.sec;
     }
 }
 
@@ -206,16 +200,17 @@ TclpGetSeconds(void)
  *----------------------------------------------------------------------
  */
 
-unsigned long
+unsigned long long
 TclpGetClicks(void)
 {
-    Tcl_WideInt usecSincePosixEpoch;
+    long long usecSincePosixEpoch;
 
-    /* Try to use high resolution timer */
-    if ( tclGetTimeProcPtr == NativeGetTime
-      && (usecSincePosixEpoch = NativeGetMicroseconds())
-    ) {
-	return (unsigned long)usecSincePosixEpoch;
+    /*
+     * Try to use high resolution timer.
+     */
+
+    if (IsTimeNative() && (usecSincePosixEpoch = NativeGetMicroseconds())) {
+	return (Tcl_WideUInt) usecSincePosixEpoch;
     } else {
 	/*
 	* Use the Tcl_GetTime abstraction to get the time in microseconds, as
@@ -224,8 +219,9 @@ TclpGetClicks(void)
 
 	Tcl_Time now;		/* Current Tcl time */
 
-	tclGetTimeProcPtr(&now, tclTimeClientData);	/* Tcl_GetTime inlined */
-	return ((unsigned long)(now.sec)*1000000UL) + (unsigned long)(now.usec);
+	GetTime(&now);
+	return ((unsigned long long)(now.sec)*1000000ULL) +
+		(unsigned long long)(now.usec);
     }
 }
 
@@ -248,7 +244,7 @@ TclpGetClicks(void)
  *----------------------------------------------------------------------
  */
 
-Tcl_WideInt
+long long
 TclpGetWideClicks(void)
 {
     LARGE_INTEGER curCounter;
@@ -261,6 +257,7 @@ TclpGetWideClicks(void)
 	 * is consistent across all processors. Therefore, the frequency need
 	 * only be queried upon application initialization.
 	 */
+
 	if (QueryPerformanceFrequency(&perfCounterFreq)) {
 	    wideClick.perfCounter = 1;
 	    wideClick.microsecsScale = 1000000.0 / perfCounterFreq.QuadPart;
@@ -274,14 +271,14 @@ TclpGetWideClicks(void)
     }
     if (wideClick.perfCounter) {
 	if (QueryPerformanceCounter(&curCounter)) {
-	    return (Tcl_WideInt)curCounter.QuadPart;
+	    return (long long)curCounter.QuadPart;
 	}
 	/* fallback using microseconds */
 	wideClick.perfCounter = 0;
 	wideClick.microsecsScale = 1;
 	return TclpGetMicroseconds();
     } else {
-    	return TclpGetMicroseconds();
+	return TclpGetMicroseconds();
     }
 }
 
@@ -295,7 +292,7 @@ TclpGetWideClicks(void)
  *	and back.
  *
  * Results:
- * 	1 click in microseconds as double.
+ *	1 click in microseconds as double.
  *
  * Side effects:
  *	None.
@@ -307,7 +304,7 @@ double
 TclpWideClickInMicrosec(void)
 {
     if (!wideClick.initialized) {
-    	(void)TclpGetWideClicks();	/* initialize */
+	(void) TclpGetWideClicks();	/* initialize */
     }
     return wideClick.microsecsScale;
 }
@@ -329,26 +326,27 @@ TclpWideClickInMicrosec(void)
  *----------------------------------------------------------------------
  */
 
-Tcl_WideInt
+long long
 TclpGetMicroseconds(void)
 {
-    Tcl_WideInt usecSincePosixEpoch;
+    long long usecSincePosixEpoch;
 
-    /* Try to use high resolution timer */
-    if ( tclGetTimeProcPtr == NativeGetTime
-      && (usecSincePosixEpoch = NativeGetMicroseconds())
-    ) {
+    /*
+     * Try to use high resolution timer.
+     */
+
+    if (IsTimeNative() && (usecSincePosixEpoch = NativeGetMicroseconds())) {
 	return usecSincePosixEpoch;
     } else {
 	/*
-	* Use the Tcl_GetTime abstraction to get the time in microseconds, as
-	* nearly as we can, and return it.
-	*/
+	 * Use the Tcl_GetTime abstraction to get the time in microseconds, as
+	 * nearly as we can, and return it.
+	 */
 
 	Tcl_Time now;
 
-	tclGetTimeProcPtr(&now, tclTimeClientData);	/* Tcl_GetTime inlined */
-	return (((Tcl_WideInt)now.sec) * 1000000) + now.usec;
+	GetTime(&now);
+	return now.sec * 1000000 + now.usec;
     }
 }
 
@@ -378,16 +376,17 @@ void
 Tcl_GetTime(
     Tcl_Time *timePtr)		/* Location to store time information. */
 {
-    Tcl_WideInt usecSincePosixEpoch;
+    long long usecSincePosixEpoch;
 
-    /* Try to use high resolution timer */
-    if ( tclGetTimeProcPtr == NativeGetTime
-      && (usecSincePosixEpoch = NativeGetMicroseconds())
-    ) {
-	timePtr->sec = (long) (usecSincePosixEpoch / 1000000);
-	timePtr->usec = (unsigned long) (usecSincePosixEpoch % 1000000);
+    /*
+     * Try to use high resolution timer.
+     */
+
+    if (IsTimeNative() && (usecSincePosixEpoch = NativeGetMicroseconds())) {
+	timePtr->sec = usecSincePosixEpoch / 1000000;
+	timePtr->usec = usecSincePosixEpoch % 1000000;
     } else {
-    	tclGetTimeProcPtr(timePtr, tclTimeClientData);
+	GetTime(timePtr);
     }
 }
 
@@ -410,14 +409,104 @@ Tcl_GetTime(
 
 static void
 NativeScaleTime(
-    Tcl_Time *timePtr,
-    ClientData clientData)
+    TCL_UNUSED(Tcl_Time *),
+    TCL_UNUSED(void *))
 {
     /*
      * Native scale is 1:1. Nothing is done.
      */
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * IsPerfCounterAvailable --
+ *
+ *	Tests whether the performance counter is available, which is a gnarly
+ *	problem on 32-bit systems. Also retrieves the nominal frequency of the
+ *	performance counter.
+ *
+ * Results:
+ *	1 if the counter is available, 0 if not.
+ *
+ * Side effects:
+ *	Updates fields of the timeInfo global. Make sure you hold the lock
+ *	before calling this.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static inline int
+IsPerfCounterAvailable(void)
+{
+    timeInfo.perfCounterAvailable =
+	    QueryPerformanceFrequency(&timeInfo.nominalFreq);
+
+    /*
+     * Some hardware abstraction layers use the CPU clock in place of the
+     * real-time clock as a performance counter reference. This results in:
+     *    - inconsistent results among the processors on multi-processor
+     *      systems.
+     *    - unpredictable changes in performance counter frequency on
+     *      "gearshift" processors such as Transmeta and SpeedStep.
+     *
+     * There seems to be no way to test whether the performance counter is
+     * reliable, but a useful heuristic is that if its frequency is 1.193182
+     * MHz or 3.579545 MHz, it's derived from a colorburst crystal and is
+     * therefore the RTC rather than the TSC.
+     *
+     * A sloppier but serviceable heuristic is that the RTC crystal is
+     * normally less than 15 MHz while the TSC crystal is virtually assured to
+     * be greater than 100 MHz. Since Win98SE appears to fiddle with the
+     * definition of the perf counter frequency (perhaps in an attempt to
+     * calibrate the clock?), we use the latter rule rather than an exact
+     * match.
+     *
+     * We also assume (perhaps questionably) that the vendors have gotten
+     * their act together on Win64, so bypass all this rubbish on that
+     * platform.
+     */
+
+#if !defined(_WIN64)
+    if (timeInfo.perfCounterAvailable &&
+	    /*
+	     * The following lines would do an exact match on crystal
+	     * frequency:
+	     *
+	     *	timeInfo.nominalFreq.QuadPart != (long long) 1193182 &&
+	     *	timeInfo.nominalFreq.QuadPart != (long long) 3579545 &&
+	     */
+	    timeInfo.nominalFreq.QuadPart > (long long) 15000000) {
+	/*
+	 * As an exception, if every logical processor on the system is on the
+	 * same chip, we use the performance counter anyway, presuming that
+	 * everyone's TSC is locked to the same oscillator.
+	 */
+
+	SYSTEM_INFO systemInfo;
+	int regs[4];
+
+	GetSystemInfo(&systemInfo);
+	if (TclWinCPUID(0, regs) == TCL_OK
+		&& regs[1] == 0x756E6547	/* "Genu" */
+		&& regs[3] == 0x49656E69	/* "ineI" */
+		&& regs[2] == 0x6C65746E	/* "ntel" */
+		&& TclWinCPUID(1, regs) == TCL_OK
+		&& ((regs[0]&0x00000F00) == 0x00000F00 /* Pentium 4 */
+		|| ((regs[0] & 0x00F00000)	/* Extended family */
+		&& (regs[3] & 0x10000000)))	/* Hyperthread */
+		&& (((regs[1]&0x00FF0000) >> 16)/* CPU count */
+			== (int)systemInfo.dwNumberOfProcessors)) {
+	    timeInfo.perfCounterAvailable = TRUE;
+	} else {
+	    timeInfo.perfCounterAvailable = FALSE;
+	}
+    }
+#endif /* above code is Win32 only */
+
+    return timeInfo.perfCounterAvailable;
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -441,18 +530,18 @@ NativeScaleTime(
  *----------------------------------------------------------------------
  */
 
-static inline Tcl_WideInt
+static inline long long
 NativeCalc100NsTicks(
     ULONGLONG fileTimeLastCall,
     LONGLONG perfCounterLastCall,
     LONGLONG curCounterFreq,
-    LONGLONG curCounter
-) {
+    LONGLONG curCounter)
+{
     return fileTimeLastCall +
-	((curCounter - perfCounterLastCall) * 10000000 / curCounterFreq);
+	    ((curCounter - perfCounterLastCall) * 10000000 / curCounterFreq);
 }
 
-static Tcl_WideInt
+static long long
 NativeGetMicroseconds(void)
 {
     /*
@@ -468,79 +557,12 @@ NativeGetMicroseconds(void)
 	    timeInfo.posixEpoch.LowPart = 0xD53E8000;
 	    timeInfo.posixEpoch.HighPart = 0x019DB1DE;
 
-	    timeInfo.perfCounterAvailable =
-		    QueryPerformanceFrequency(&timeInfo.nominalFreq);
-
-	    /*
-	     * Some hardware abstraction layers use the CPU clock in place of
-	     * the real-time clock as a performance counter reference. This
-	     * results in:
-	     *    - inconsistent results among the processors on
-	     *      multi-processor systems.
-	     *    - unpredictable changes in performance counter frequency on
-	     *      "gearshift" processors such as Transmeta and SpeedStep.
-	     *
-	     * There seems to be no way to test whether the performance
-	     * counter is reliable, but a useful heuristic is that if its
-	     * frequency is 1.193182 MHz or 3.579545 MHz, it's derived from a
-	     * colorburst crystal and is therefore the RTC rather than the
-	     * TSC.
-	     *
-	     * A sloppier but serviceable heuristic is that the RTC crystal is
-	     * normally less than 15 MHz while the TSC crystal is virtually
-	     * assured to be greater than 100 MHz. Since Win98SE appears to
-	     * fiddle with the definition of the perf counter frequency
-	     * (perhaps in an attempt to calibrate the clock?), we use the
-	     * latter rule rather than an exact match.
-	     *
-	     * We also assume (perhaps questionably) that the vendors have
-	     * gotten their act together on Win64, so bypass all this rubbish
-	     * on that platform.
-	     */
-
-#if !defined(_WIN64)
-	    if (timeInfo.perfCounterAvailable
-		    /*
-		     * The following lines would do an exact match on crystal
-		     * frequency:
-		     * && timeInfo.nominalFreq.QuadPart != (Tcl_WideInt)1193182
-		     * && timeInfo.nominalFreq.QuadPart != (Tcl_WideInt)3579545
-		     */
-		    && timeInfo.nominalFreq.QuadPart > (Tcl_WideInt) 15000000){
-		/*
-		 * As an exception, if every logical processor on the system
-		 * is on the same chip, we use the performance counter anyway,
-		 * presuming that everyone's TSC is locked to the same
-		 * oscillator.
-		 */
-
-		SYSTEM_INFO systemInfo;
-		unsigned int regs[4];
-
-		GetSystemInfo(&systemInfo);
-		if (TclWinCPUID(0, regs) == TCL_OK
-			&& regs[1] == 0x756E6547	/* "Genu" */
-			&& regs[3] == 0x49656E69	/* "ineI" */
-			&& regs[2] == 0x6C65746E	/* "ntel" */
-			&& TclWinCPUID(1, regs) == TCL_OK
-			&& ((regs[0]&0x00000F00) == 0x00000F00 /* Pentium 4 */
-			|| ((regs[0] & 0x00F00000)	/* Extended family */
-			&& (regs[3] & 0x10000000)))	/* Hyperthread */
-			&& (((regs[1]&0x00FF0000) >> 16)/* CPU count */
-			    == systemInfo.dwNumberOfProcessors)) {
-		    timeInfo.perfCounterAvailable = TRUE;
-		} else {
-		    timeInfo.perfCounterAvailable = FALSE;
-		}
-	    }
-#endif /* above code is Win32 only */
-
 	    /*
 	     * If the performance counter is available, start a thread to
 	     * calibrate it.
 	     */
 
-	    if (timeInfo.perfCounterAvailable) {
+	    if (IsPerfCounterAvailable()) {
 		DWORD id;
 
 		InitializeCriticalSection(&timeInfo.cs);
@@ -574,8 +596,8 @@ NativeGetMicroseconds(void)
 
 	ULONGLONG fileTimeLastCall;
 	LONGLONG perfCounterLastCall, curCounterFreq;
-				/* Copy with current data of calibration cycle */
-
+				/* Copy with current data of calibration
+				 * cycle. */
 	LARGE_INTEGER curCounter;
 				/* Current performance counter. */
 
@@ -584,6 +606,7 @@ NativeGetMicroseconds(void)
 	/*
 	 * Hold time section locked as short as possible
 	 */
+
 	EnterCriticalSection(&timeInfo.cs);
 
 	fileTimeLastCall = timeInfo.fileTimeLastCall.QuadPart;
@@ -615,17 +638,21 @@ NativeGetMicroseconds(void)
 	 */
 
 	if (curCounter.QuadPart - perfCounterLastCall <
-		11 * curCounterFreq * timeInfo.calibrationInterv / 10
-	) {
-	    /* Calibrated file-time is saved from Posix in 100-ns ticks */
+		11 * curCounterFreq * timeInfo.calibrationInterv / 10) {
+	    /*
+	     * Calibrated file-time is saved from Posix in 100-ns ticks.
+	     */
+
 	    return NativeCalc100NsTicks(fileTimeLastCall,
-		perfCounterLastCall, curCounterFreq, curCounter.QuadPart) / 10;
+		    perfCounterLastCall, curCounterFreq,
+		    curCounter.QuadPart) / 10;
 	}
     }
 
     /*
      * High resolution timer is not available.
      */
+
     return 0;
 }
 
@@ -649,25 +676,27 @@ NativeGetMicroseconds(void)
 static void
 NativeGetTime(
     Tcl_Time *timePtr,
-    ClientData clientData)
+    TCL_UNUSED(void *))
 {
-    Tcl_WideInt usecSincePosixEpoch;
+    long long usecSincePosixEpoch;
 
     /*
      * Try to use high resolution timer.
      */
-    if ( (usecSincePosixEpoch = NativeGetMicroseconds()) ) {
-	timePtr->sec = (long) (usecSincePosixEpoch / 1000000);
-	timePtr->usec = (unsigned long) (usecSincePosixEpoch % 1000000);
+
+    usecSincePosixEpoch = NativeGetMicroseconds();
+    if (usecSincePosixEpoch) {
+	timePtr->sec = usecSincePosixEpoch / 1000000;
+	timePtr->usec = usecSincePosixEpoch % 1000000;
     } else {
 	/*
-	* High resolution timer is not available. Just use ftime.
-	*/
+	 * High resolution timer is not available. Just use ftime.
+	 */
 
 	struct _timeb t;
 
 	_ftime(&t);
-	timePtr->sec = (long)t.time;
+	timePtr->sec = t.time;
 	timePtr->usec = t.millitm * 1000;
     }
 }
@@ -694,7 +723,7 @@ void TclWinResetTimerResolution(void);
 
 static void
 StopCalibration(
-    ClientData unused)		/* Client data is unused */
+    TCL_UNUSED(void *))
 {
     SetEvent(timeInfo.exitEvent);
 
@@ -708,241 +737,6 @@ StopCalibration(
     CloseHandle(timeInfo.calibrationThread);
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TclpGetDate --
- *
- *	This function converts between seconds and struct tm. If useGMT is
- *	true, then the returned date will be in Greenwich Mean Time (GMT).
- *	Otherwise, it will be in the local time zone.
- *
- * Results:
- *	Returns a static tm structure.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-struct tm *
-TclpGetDate(
-    const time_t *t,
-    int useGMT)
-{
-    struct tm *tmPtr;
-    time_t time;
-#if defined(_WIN64) || (defined(_USE_64BIT_TIME_T) || (defined(_MSC_VER) && _MSC_VER < 1400))
-#   define  t2 *t /* no need to cripple time to 32-bit */
-#else
-    time_t t2 = *(__time32_t *)t;
-#endif
-
-    if (!useGMT) {
-#if defined(_MSC_VER) && (_MSC_VER >= 1900)
-#	undef timezone /* prevent conflict with timezone() function */
-	long timezone = 0;
-#endif
-
-	tzset();
-
-	/*
-	 * If we are in the valid range, let the C run-time library handle it.
-	 * Otherwise we need to fake it. Note that this algorithm ignores
-	 * daylight savings time before the epoch.
-	 */
-
-	/*
-	 * Hmm, Borland's localtime manages to return NULL under certain
-	 * circumstances (e.g. wintime.test, test 1.2). Nobody tests for this,
-	 * since 'localtime' isn't supposed to do this, possibly leading to
-	 * crashes.
-	 *
-	 * Patch: We only call this function if we are at least one day into
-	 * the epoch, else we handle it ourselves (like we do for times < 0).
-	 * H. Giese, June 2003
-	 */
-
-#ifdef __BORLANDC__
-#define LOCALTIME_VALIDITY_BOUNDARY	SECSPERDAY
-#else
-#define LOCALTIME_VALIDITY_BOUNDARY	0
-#endif
-
-	if (t2 >= LOCALTIME_VALIDITY_BOUNDARY) {
-	    return TclpLocaltime(&t2);
-	}
-
-#if defined(_MSC_VER) && (_MSC_VER >= 1900)
-	_get_timezone(&timezone);
-#endif
-
-	time = t2 - timezone;
-
-	/*
-	 * If we aren't near to overflowing the long, just add the bias and
-	 * use the normal calculation. Otherwise we will need to adjust the
-	 * result at the end.
-	 */
-
-	if (t2 < (LONG_MAX - 2*SECSPERDAY) && t2 > (LONG_MIN + 2*SECSPERDAY)) {
-	    tmPtr = ComputeGMT(&time);
-	} else {
-	    tmPtr = ComputeGMT(&t2);
-
-	    tzset();
-
-	    /*
-	     * Add the bias directly to the tm structure to avoid overflow.
-	     * Propagate seconds overflow into minutes, hours and days.
-	     */
-
-	    time = tmPtr->tm_sec - timezone;
-	    tmPtr->tm_sec = (int)(time % 60);
-	    if (tmPtr->tm_sec < 0) {
-		tmPtr->tm_sec += 60;
-		time -= 60;
-	    }
-
-	    time = tmPtr->tm_min + time/60;
-	    tmPtr->tm_min = (int)(time % 60);
-	    if (tmPtr->tm_min < 0) {
-		tmPtr->tm_min += 60;
-		time -= 60;
-	    }
-
-	    time = tmPtr->tm_hour + time/60;
-	    tmPtr->tm_hour = (int)(time % 24);
-	    if (tmPtr->tm_hour < 0) {
-		tmPtr->tm_hour += 24;
-		time -= 24;
-	    }
-
-	    time /= 24;
-	    tmPtr->tm_mday += (int)time;
-	    tmPtr->tm_yday += (int)time;
-	    tmPtr->tm_wday = (tmPtr->tm_wday + (int)time) % 7;
-	}
-    } else {
-	tmPtr = ComputeGMT(&t2);
-    }
-    return tmPtr;
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * ComputeGMT --
- *
- *	This function computes GMT given the number of seconds since the epoch
- *	(midnight Jan 1 1970).
- *
- * Results:
- *	Returns a (per thread) statically allocated struct tm.
- *
- * Side effects:
- *	Updates the values of the static struct tm.
- *
- *----------------------------------------------------------------------
- */
-
-static struct tm *
-ComputeGMT(
-    const time_t *tp)
-{
-    struct tm *tmPtr;
-    long tmp, rem;
-    int isLeap;
-    const int *days;
-    ThreadSpecificData *tsdPtr = TCL_TSD_INIT(&dataKey);
-
-    tmPtr = &tsdPtr->tm;
-
-    /*
-     * Compute the 4 year span containing the specified time.
-     */
-
-    tmp = (long)(*tp / SECSPER4YEAR);
-    rem = (long)(*tp % SECSPER4YEAR);
-
-    /*
-     * Correct for weird mod semantics so the remainder is always positive.
-     */
-
-    if (rem < 0) {
-	tmp--;
-	rem += SECSPER4YEAR;
-    }
-
-    /*
-     * Compute the year after 1900 by taking the 4 year span and adjusting for
-     * the remainder. This works because 2000 is a leap year, and 1900/2100
-     * are out of the range.
-     */
-
-    tmp = (tmp * 4) + 70;
-    isLeap = 0;
-    if (rem >= SECSPERYEAR) {			  /* 1971, etc. */
-	tmp++;
-	rem -= SECSPERYEAR;
-	if (rem >= SECSPERYEAR) {		  /* 1972, etc. */
-	    tmp++;
-	    rem -= SECSPERYEAR;
-	    if (rem >= SECSPERYEAR + SECSPERDAY) { /* 1973, etc. */
-		tmp++;
-		rem -= SECSPERYEAR + SECSPERDAY;
-	    } else {
-		isLeap = 1;
-	    }
-	}
-    }
-    tmPtr->tm_year = tmp;
-
-    /*
-     * Compute the day of year and leave the seconds in the current day in the
-     * remainder.
-     */
-
-    tmPtr->tm_yday = rem / SECSPERDAY;
-    rem %= SECSPERDAY;
-
-    /*
-     * Compute the time of day.
-     */
-
-    tmPtr->tm_hour = rem / 3600;
-    rem %= 3600;
-    tmPtr->tm_min = rem / 60;
-    tmPtr->tm_sec = rem % 60;
-
-    /*
-     * Compute the month and day of month.
-     */
-
-    days = (isLeap) ? leapDays : normalDays;
-    for (tmp = 1; days[tmp] < tmPtr->tm_yday; tmp++) {
-	/* empty body */
-    }
-    tmPtr->tm_mon = --tmp;
-    tmPtr->tm_mday = tmPtr->tm_yday - days[tmp];
-
-    /*
-     * Compute day of week.  Epoch started on a Thursday.
-     */
-
-    tmPtr->tm_wday = (long)(*tp / SECSPERDAY) + 4;
-    if ((*tp % SECSPERDAY) < 0) {
-	tmPtr->tm_wday--;
-    }
-    tmPtr->tm_wday %= 7;
-    if (tmPtr->tm_wday < 0) {
-	tmPtr->tm_wday += 7;
-    }
-
-    return tmPtr;
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -971,7 +765,7 @@ ComputeGMT(
 
 static DWORD WINAPI
 CalibrationThread(
-    LPVOID arg)
+    TCL_UNUSED(LPVOID))
 {
     FILETIME curFileTime;
     DWORD waitResult;
@@ -985,7 +779,11 @@ CalibrationThread(
     QueryPerformanceFrequency(&timeInfo.curCounterFreq);
     timeInfo.fileTimeLastCall.LowPart = curFileTime.dwLowDateTime;
     timeInfo.fileTimeLastCall.HighPart = curFileTime.dwHighDateTime;
-    /* Calibrated file-time will be saved from Posix in 100-ns ticks */
+
+    /*
+     * Calibrated file-time will be saved from Posix in 100-ns ticks.
+     */
+
     timeInfo.fileTimeLastCall.QuadPart -= timeInfo.posixEpoch.QuadPart;
 
     ResetCounterSamples(timeInfo.fileTimeLastCall.QuadPart,
@@ -1049,12 +847,12 @@ UpdateTimeEachSecond(void)
 				/* File time of the previous calibration */
     LARGE_INTEGER curFileTime;	/* File time at the time this callback was
 				 * scheduled. */
-    Tcl_WideInt estFreq;	/* Estimated perf counter frequency. */
-    Tcl_WideInt vt0;		/* Tcl time right now. */
-    Tcl_WideInt vt1;		/* Tcl time one second from now. */
-    Tcl_WideInt tdiff;		/* Difference between system clock and Tcl
+    long long estFreq;		/* Estimated perf counter frequency. */
+    long long vt0;		/* Tcl time right now. */
+    long long vt1;		/* Tcl time one second from now. */
+    long long tdiff;		/* Difference between system clock and Tcl
 				 * time. */
-    Tcl_WideInt driftFreq;	/* Frequency needed to drift virtual time into
+    long long driftFreq;	/* Frequency needed to drift virtual time into
 				 * step over 1 second. */
 
     /*
@@ -1065,12 +863,17 @@ UpdateTimeEachSecond(void)
     curFileTime.LowPart = curSysTime.dwLowDateTime;
     curFileTime.HighPart = curSysTime.dwHighDateTime;
     curFileTime.QuadPart -= timeInfo.posixEpoch.QuadPart;
-    /* If calibration still not needed (check for possible time switch) */
-    if ( curFileTime.QuadPart > lastFileTime.QuadPart
-      && curFileTime.QuadPart < lastFileTime.QuadPart +
-      				    (timeInfo.calibrationInterv * 10000000)
-    ) {
-    	/* again in next one second */
+
+    /*
+     * If calibration still not needed (check for possible time switch)
+     */
+
+    if (curFileTime.QuadPart > lastFileTime.QuadPart && curFileTime.QuadPart <
+	    lastFileTime.QuadPart + (timeInfo.calibrationInterv * 10000000)) {
+	/*
+	 * Look again in next one second.
+	 */
+
 	return;
     }
     QueryPerformanceCounter(&curPerfCounter);
@@ -1106,7 +909,7 @@ UpdateTimeEachSecond(void)
      */
 
     estFreq = AccumulateSample(curPerfCounter.QuadPart,
-	    (Tcl_WideUInt) curFileTime.QuadPart);
+	    (unsigned long long) curFileTime.QuadPart);
 
     /*
      * We want to adjust things so that time appears to be continuous.
@@ -1126,8 +929,9 @@ UpdateTimeEachSecond(void)
      */
 
     vt0 = NativeCalc100NsTicks(timeInfo.fileTimeLastCall.QuadPart,
-	    timeInfo.perfCounterLastCall.QuadPart, timeInfo.curCounterFreq.QuadPart,
-	    curPerfCounter.QuadPart);
+	    timeInfo.perfCounterLastCall.QuadPart,
+	    timeInfo.curCounterFreq.QuadPart, curPerfCounter.QuadPart);
+
     /*
      * If we've gotten more than a second away from system time, then drifting
      * the clock is going to be pretty hopeless. Just let it jump. Otherwise,
@@ -1136,17 +940,25 @@ UpdateTimeEachSecond(void)
 
     tdiff = vt0 - curFileTime.QuadPart;
     if (tdiff > 10000000 || tdiff < -10000000) {
-    	/* jump to current system time, use curent estimated frequency */
-    	vt0 = curFileTime.QuadPart;
+	/*
+	 * Jump to current system time, use curent estimated frequency.
+	 */
+
+	vt0 = curFileTime.QuadPart;
     } else {
-    	/* calculate new frequency and estimate drift to the next second */
+	/*
+	 * Calculate new frequency and estimate drift to the next second.
+	 */
+
 	vt1 = 20000000 + curFileTime.QuadPart;
 	driftFreq = (estFreq * 20000000 / (vt1 - vt0));
+
 	/*
-	 * Avoid too large drifts (only half of the current difference),
-	 * that allows also be more accurate (aspire to the smallest tdiff),
-	 * so then we can prolong calibration interval by tdiff < 100000
+	 * Avoid too large drifts (only half of the current difference), that
+	 * allows also be more accurate (aspire to the smallest tdiff), so
+	 * then we can prolong calibration interval by tdiff < 100000
 	 */
+
 	driftFreq = timeInfo.curCounterFreq.QuadPart +
 		(driftFreq - timeInfo.curCounterFreq.QuadPart) / 2;
 
@@ -1154,50 +966,79 @@ UpdateTimeEachSecond(void)
 	 * Average between estimated, 2 current and 5 drifted frequencies,
 	 * (do the soft drifting as possible)
 	 */
-	estFreq = (estFreq + 2 * timeInfo.curCounterFreq.QuadPart + 5 * driftFreq) / 8;
+
+	estFreq = (estFreq + 2 * timeInfo.curCounterFreq.QuadPart +
+		5 * driftFreq) / 8;
     }
 
-    /* Avoid too large discrepancy from nominal frequency */
-    if (estFreq > 1003*timeInfo.nominalFreq.QuadPart/1000) {
-	estFreq = 1003*timeInfo.nominalFreq.QuadPart/1000;
+    /*
+     * Avoid too large discrepancy from nominal frequency.
+     */
+
+    if (estFreq > 1003 * timeInfo.nominalFreq.QuadPart / 1000) {
+	estFreq = 1003 * timeInfo.nominalFreq.QuadPart / 1000;
 	vt0 = curFileTime.QuadPart;
-    } else if (estFreq < 997*timeInfo.nominalFreq.QuadPart/1000) {
-	estFreq = 997*timeInfo.nominalFreq.QuadPart/1000;
+    } else if (estFreq < 997 * timeInfo.nominalFreq.QuadPart / 1000) {
+	estFreq = 997 * timeInfo.nominalFreq.QuadPart / 1000;
 	vt0 = curFileTime.QuadPart;
     } else if (vt0 != curFileTime.QuadPart) {
 	/*
-	 * Be sure the clock ticks never backwards (avoid it by negative drifting)
-	 * just compare native time (in 100-ns) before and hereafter using
-	 * new calibrated values) and do a small adjustment (short time freeze)
+	 * Be sure the clock ticks never backwards (avoid it by negative
+	 * drifting). Just compare native time (in 100-ns) before and
+	 * hereafter using new calibrated values) and do a small adjustment
+	 * (short time freeze).
 	 */
+
 	LARGE_INTEGER newPerfCounter;
-	Tcl_WideInt nt0, nt1;
+	long long nt0, nt1;
 
 	QueryPerformanceCounter(&newPerfCounter);
 	nt0 = NativeCalc100NsTicks(timeInfo.fileTimeLastCall.QuadPart,
-		timeInfo.perfCounterLastCall.QuadPart, timeInfo.curCounterFreq.QuadPart,
-		newPerfCounter.QuadPart);
+		timeInfo.perfCounterLastCall.QuadPart,
+		timeInfo.curCounterFreq.QuadPart, newPerfCounter.QuadPart);
 	nt1 = NativeCalc100NsTicks(vt0,
-		curPerfCounter.QuadPart, estFreq,
-		newPerfCounter.QuadPart);
-	if (nt0 > nt1) { /* drifted backwards, try to compensate with new base */
-	    /* first adjust with a micro jump (short frozen time is acceptable) */
+		curPerfCounter.QuadPart, estFreq, newPerfCounter.QuadPart);
+	if (nt0 > nt1) {
+	    /*
+	     * Drifted backwards, try to compensate with new base.
+	     *
+	     * First adjust with a micro jump (short frozen time is
+	     * acceptable).
+	     */
+
 	    vt0 += nt0 - nt1;
-	    /* if drift unavoidable (e. g. we had a time switch), then reset it */
+
+	    /*
+	     * If drift unavoidable (e. g. we had a time switch), then reset
+	     * it.
+	     */
+
 	    vt1 = vt0 - curFileTime.QuadPart;
 	    if (vt1 > 10000000 || vt1 < -10000000) {
-	    	/* larger jump resp. shift relative new file-time */
-	    	vt0 = curFileTime.QuadPart;
+		/*
+		 * Larger jump resp. shift relative new file-time.
+		 */
+
+		vt0 = curFileTime.QuadPart;
 	    }
 	}
     }
 
-    /* In lock commit new values to timeInfo (hold lock as short as possible) */
+    /*
+     * In lock commit new values to timeInfo (hold lock as short as possible)
+     */
+
     EnterCriticalSection(&timeInfo.cs);
 
-    /* grow calibration interval up to 10 seconds (if still precise enough) */
+    /*
+     * Grow calibration interval up to 10 seconds (if still precise enough)
+     */
+
     if (tdiff < -100000 || tdiff > 100000) {
-	/* too long drift - reset calibration interval to 1000 second */
+	/*
+	 * Too long drift. Reset calibration interval to 1000 second.
+	 */
+
 	timeInfo.calibrationInterv = 1;
     } else if (timeInfo.calibrationInterv < 10) {
 	timeInfo.calibrationInterv++;
@@ -1231,12 +1072,13 @@ UpdateTimeEachSecond(void)
 
 static void
 ResetCounterSamples(
-    Tcl_WideUInt fileTime,	/* Current file time */
-    Tcl_WideInt perfCounter,	/* Current performance counter */
-    Tcl_WideInt perfFreq)	/* Target performance frequency */
+    unsigned long long fileTime,/* Current file time */
+    long long perfCounter,	/* Current performance counter */
+    long long perfFreq)		/* Target performance frequency */
 {
     int i;
-    for (i=SAMPLES-1 ; i>=0 ; --i) {
+
+    for (i = SAMPLES - 1 ; i >= 0 ; --i) {
 	timeInfo.perfCounterSample[i] = perfCounter;
 	timeInfo.fileTimeSample[i] = fileTime;
 	perfCounter -= perfFreq;
@@ -1271,20 +1113,22 @@ ResetCounterSamples(
  * case).
  */
 
-static Tcl_WideInt
+static long long
 AccumulateSample(
-    Tcl_WideInt perfCounter,
-    Tcl_WideUInt fileTime)
+    long long perfCounter,
+    unsigned long long fileTime)
 {
-    Tcl_WideUInt workFTSample;	/* File time sample being removed from or
+    unsigned long long workFTSample;
+				/* File time sample being removed from or
 				 * added to the circular buffer. */
-    Tcl_WideInt workPCSample;	/* Performance counter sample being removed
+    long long workPCSample;	/* Performance counter sample being removed
 				 * from or added to the circular buffer. */
-    Tcl_WideUInt lastFTSample;	/* Last file time sample recorded */
-    Tcl_WideInt lastPCSample;	/* Last performance counter sample recorded */
-    Tcl_WideInt FTdiff;		/* Difference between last FT and current */
-    Tcl_WideInt PCdiff;		/* Difference between last PC and current */
-    Tcl_WideInt estFreq;	/* Estimated performance counter frequency */
+    unsigned long long lastFTSample;
+				/* Last file time sample recorded */
+    long long lastPCSample;	/* Last performance counter sample recorded */
+    long long FTdiff;		/* Difference between last FT and current */
+    long long PCdiff;		/* Difference between last PC and current */
+    long long estFreq;		/* Estimated performance counter frequency */
 
     /*
      * Test for jumps and reset the samples if we have one.
@@ -1318,7 +1162,7 @@ AccumulateSample(
 	estFreq = 10000000 * (perfCounter - workPCSample)
 		/ (fileTime - workFTSample);
 	timeInfo.perfCounterSample[timeInfo.sampleNo] = perfCounter;
-	timeInfo.fileTimeSample[timeInfo.sampleNo] = (Tcl_WideInt) fileTime;
+	timeInfo.fileTimeSample[timeInfo.sampleNo] = (long long) fileTime;
 
 	/*
 	 * Advance the sample number.
@@ -1332,75 +1176,6 @@ AccumulateSample(
     }
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TclpGmtime --
- *
- *	Wrapper around the 'gmtime' library function to make it thread safe.
- *
- * Results:
- *	Returns a pointer to a 'struct tm' in thread-specific data.
- *
- * Side effects:
- *	Invokes gmtime or gmtime_r as appropriate.
- *
- *----------------------------------------------------------------------
- */
-
-struct tm *
-TclpGmtime(
-    const time_t *timePtr)	/* Pointer to the number of seconds since the
-				 * local system's epoch */
-{
-    /*
-     * The MS implementation of gmtime is thread safe because it returns the
-     * time in a block of thread-local storage, and Windows does not provide a
-     * Posix gmtime_r function.
-     */
-
-#if defined(_WIN64) || defined(_USE_64BIT_TIME_T) || (defined(_MSC_VER) && _MSC_VER < 1400)
-    return gmtime(timePtr);
-#else
-    return _gmtime32((const __time32_t *)timePtr);
-#endif
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * TclpLocaltime --
- *
- *	Wrapper around the 'localtime' library function to make it thread
- *	safe.
- *
- * Results:
- *	Returns a pointer to a 'struct tm' in thread-specific data.
- *
- * Side effects:
- *	Invokes localtime or localtime_r as appropriate.
- *
- *----------------------------------------------------------------------
- */
-
-struct tm *
-TclpLocaltime(
-    const time_t *timePtr)	/* Pointer to the number of seconds since the
-				 * local system's epoch */
-{
-    /*
-     * The MS implementation of localtime is thread safe because it returns
-     * the time in a block of thread-local storage, and Windows does not
-     * provide a Posix localtime_r function.
-     */
-
-#if defined(_WIN64) || defined(_USE_64BIT_TIME_T) || (defined(_MSC_VER) && _MSC_VER < 1400)
-    return localtime(timePtr);
-#else
-    return _localtime32((const __time32_t *)timePtr);
-#endif
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -1422,7 +1197,7 @@ void
 Tcl_SetTimeProc(
     Tcl_GetTimeProc *getProc,
     Tcl_ScaleTimeProc *scaleProc,
-    ClientData clientData)
+    void *clientData)
 {
     tclGetTimeProcPtr = getProc;
     tclScaleTimeProcPtr = scaleProc;
@@ -1449,7 +1224,7 @@ void
 Tcl_QueryTimeProc(
     Tcl_GetTimeProc **getProc,
     Tcl_ScaleTimeProc **scaleProc,
-    ClientData *clientData)
+    void **clientData)
 {
     if (getProc) {
 	*getProc = tclGetTimeProcPtr;
